// EMDataC.C - Steve Ludtke  11/27/2000
// More stuff for the bloated EMData object
// These routines are mainly for treating EMData objects as
// vectors and matrices. Inefficient, but functional.

#include <stdio.h>

#ifndef _WIN32
 #include <unistd.h>
#else
 #include "UnixToWin.h"
#endif

#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "List.h"
#include "mrc.h"
#include "imagic.h"
#include "emim.h"
#include "gatan.h"
#include "util.h"
#include "EMData.h"

void EMData::cutSlice(EMData *map,float DZ,Euler *ort,int interp,float DX,float DY) {
float *sdata = map->getDataRO();
float *ddata = getData();
Euler Ort(0,0,0);
if (!ort) ort=&Ort;
float *mx = ort->mat();

int Mx=map->xSize();
int My=map->ySize();
int Mz=map->zSize();

int x,y,x2,y2;
float xx,yy,zz;
for (y=0; y<ySize(); y++) {
	y2=y-ySize()/2-DY;
	for (x=0; x<xSize(); x++) {
		x2=x-xSize()/2-DX;
				
		xx=x2*mx[0]+y2*mx[1]+DZ*mx[2]+Mx/2;
		yy=x2*mx[3]+y2*mx[4]+DZ*mx[5]+My/2;
		zz=x2*mx[6]+y2*mx[7]+DZ*mx[8]+Mz/2;

		if (xx<0||yy<0||zz<0 || xx>Mx-2||yy>My-2||zz>Mz-2) {
			ddata[x+y*xSize()]=0;
			continue;
		}

		float t=xx-floor(xx);
		float u=yy-floor(yy);
		float v=zz-floor(zz);
		
		if (interp) {
			int k=(int)(floor(xx)+floor(yy)*Mx+floor(zz)*Mx*My);

			ddata[x+y*xSize()]=trilin(sdata[k],sdata[k+1],sdata[k+Mx],
				sdata[k+Mx+1],sdata[k+Mx*My],sdata[k+1+Mx*My],sdata[k+Mx+Mx*My],
				sdata[k+1+Mx+Mx*My],t,u,v);
		}
		else {
			int k=(int)(round(xx)+round(yy)*Mx+round(zz)*Mx*My);
			ddata[x+y*xSize()]=sdata[k];
		}
	}
}

doneData();
map->doneData();

}

void EMData::uncutSlice(EMData *map,float DZ,Euler *ort,float DX,float DY) {
float *ddata = map->getDataRO();
float *sdata = getData();
Euler Ort(0,0,0);
if (!ort) ort=&Ort;
float *mx = ort->mat();

int Mx=map->xSize();
int My=map->ySize();
int Mz=map->zSize();

int x,y,x2,y2;
float xx,yy,zz;
for (y=0; y<ySize(); y++) {
	y2=y-ySize()/2-DY;
	for (x=0; x<xSize(); x++) {
		x2=x-xSize()/2-DX;
				
		xx=x2*mx[0]+y2*mx[1]+DZ*mx[2]+Mx/2;
		yy=x2*mx[3]+y2*mx[4]+DZ*mx[5]+My/2;
		zz=x2*mx[6]+y2*mx[7]+DZ*mx[8]+Mz/2;
		if (xx<0||yy<0||zz<0 || xx>Mx-2||yy>My-2||zz>Mz-2) continue;
		
		int k=round(xx)+round(yy)*Mx+round(zz)*Mx*My;
		ddata[k]=sdata[x+y*xSize()];
		
	}
}

doneData();
map->doneData();
}


// returns an EMData object containing all of the data at y=n
EMData *EMData::row(int n) {
EMData *ret = new EMData;
ret->setSize(xSize(),1,1);
memcpy(ret->getData(),getDataRO()+xSize()*n,xSize()*sizeof(float));
doneData();
ret->doneData();
return ret;
}

// returns an EMData object containing all of the data at x=n
EMData *EMData::col(int n) {
EMData *ret = new EMData;
ret->setSize(ySize(),1,1);
float *dst=ret->getData(),*src=getDataRO();
for (int i; i<ySize(); i++) dst[i]=src[i*xSize()+n];
ret->doneData();
doneData();
return ret;
}

void EMData::setRow(EMData *d,int n) {
float *dst=getData(),*src=d->getDataRO();

memcpy(dst+xSize()*n,src,xSize()*sizeof(float));
doneData();
d->doneData();
}

void EMData::setCol(EMData *d,int n) {
float *dst=getData(),*src=d->getDataRO();
for (int i; i<ySize(); i++) dst[i*xSize()+n]=src[i];
d->doneData();
doneData();
}


// UNIMPLEMENTED
// 2d matrix multiply  this * second
// Note both row and column vectors (if vector matrix mult) stored
// as nx>1 ny=1
EMData *EMData::mxMult(EMData *second) {
return NULL;
}

// Distance between 2 vectors, n specifies the row in 'second'
float EMData::dist(EMData *second,int n) {
if (second->xSize()!=xSize() || ySize()!=1) return -1.0;
float ret=0;
if (n>second->ySize()||n<0) return -1.0;
float *d1 = getDataRO(), *d2 = second->getDataRO() + second->xSize()*n;
for (int i=0; i<xSize(); i++) ret+=SQR(d1[i]-d2[i]);
return sqrt(ret);
}
