#include "Euler.h"
#include "Vect.h"
#include "VolFrag.h"
#include "Cylinder.h"
#include "PyUtil.h"
#include "PyEMData.h"
#include "PyPickle.h"
#include "PyEuler.h"
#include "PyVect.h"
#include <boost/python.hpp>
#include <boost/python/to_python_converter.hpp>


namespace python = boost::python;

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(print_overloads, print, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Euler_printMatrix, printMatrix, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Euler_diff, diff, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Euler_setAngle1, setAngle, 3, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Euler_setAngle2, setAngle, 4, 5);

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Vect_angle, Vect::angle, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinder_toInventor, toInventor, 0, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinder_toDejaVu, toDejaVu, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinder_toCosec, toCosec, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinder_if_merge, if_merge, 1, 3);

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinders_WriteDejaVu, WriteDejaVu, 1, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinders_WriteCosec, WriteCosec, 1, 7);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(Cylinders_OrientationDiff, OrientationDiff, 1, 3);


void add_Euler()
{
    python::class_<Euler> Euler_class("_Euler", "Don't use this class. Use 'Euler' instead.");
    
    python::class_<PyEuler, python::bases<Euler> > PyEuler_class("Euler", "Euler();\nEuler(float a1, float a2, float a3); \nEuler(float a1, float a2, float a3, EulerType type); \nEuler(float e1, float e2, float e3, float e0, EulerType type);");

    PyEuler_class.def(python::init<>());
    PyEuler_class.def(python::init<float, float, float>());
    PyEuler_class.def(python::init<float, float, float, int>());
    PyEuler_class.def(python::init<float, float, float, float, int>());
    PyEuler_class.def(python::init<PyList>());
    
    PyEuler_class.def(python::self * python::self);
    PyEuler_class.def(python::self / python::self);
    PyEuler_class.def(python::self != python::self);
    PyEuler_class.def(python::self == python::self);
    
    PyEuler_class.def("getByType", &PyEuler::getByType,
		      "tuple getByType(EulerType type) \n  Return Euler values in a tuple given a type.");
    
    PyEuler_class.def("setByType", &PyEuler::setByType,
		      "setByType(tuple value, EulerType type) \n  Set Euler values given a tuple and the type.");

    
    Euler_class.def("init", &Euler::init, "init() \n  initialize an Euler instance.");
    Euler_class.def("rectify", &Euler::rectify, "rectify() \n  rectify an Euler instance.");
    Euler_class.def("increment", &Euler::increment,
		    "increment(float delta_alt, float delta_az, float del_phi) \n  increment an Euler in place");
    Euler_class.def("diff", &Euler::diff,
		    Euler_diff("float diff(Euler angle, bool nophi=true) \n  find angle between two direction vectors"));

    Euler_class.def("inverse", &Euler::inverse, "Euler inverse() \n  takes the inverse of an Euler instance");

    Euler_class.def("EulerMult", &Euler::EulerMult,
		    "Euler EulerMult(Euler bc) \n  returns the value of the given rotation");
    Euler_class.def("getMaxSymEl", &Euler::getMaxSymEl,
		    "int getMaxSymEl() \n  gets the order of the group");
    Euler_class.def("resetSym", &Euler::resetSym,
		    "resetSym() \n  initializes symcounter_ without ing 1st element");
    Euler_class.def("FirstSym", &Euler::FirstSym,
		    "Euler FirstSym() \n  initializes");
    Euler_class.def("NextSym", &Euler::NextSym,
		    "Euler NextSym()");
    Euler_class.def("SymN", &Euler::SymN,
		    "Euler SymN(n) \n  returns the n'th symmetric element");
    Euler_class.def("valid", &Euler::valid,
		    "int valid()");
    Euler_class.def("fromMRC", &Euler::fromMRC,
		    "fromMRC(theta, phi, omega) \n  set from MRC angles ");
    Euler_class.def("thetaMRC", &Euler::thetaMRC,
		    "float thetaMRC()");
    Euler_class.def("phiMRC", &Euler::phiMRC, "float phiMRC()");
    Euler_class.def("omegaMRC", &Euler::omegaMRC, "float omegaMRC()");
    Euler_class.def("alt", &Euler::alt, "float alt()");
    Euler_class.def("az", &Euler::az, "float az()");
    Euler_class.def("phi", &Euler::phi, "float phi()");
    Euler_class.def("alpha", &Euler::alpha, "float alpha()");
    Euler_class.def("beta", &Euler::beta, "float beta()");
    Euler_class.def("gamma", &Euler::gamma, "float gamma()");
    Euler_class.def("phiSpider", &Euler::phiSpider, "float phiSpider()");
    Euler_class.def("thetaSpider", &Euler::thetaSpider, "float thetaSpider()");
    Euler_class.def("gammaSpider", &Euler::gammaSpider, "float gammaSpider()");
    Euler_class.def("e0", &Euler::e0, "float e0()");
    Euler_class.def("e1", &Euler::e1, "float e1()");
    Euler_class.def("e2", &Euler::e2, "float e2()");
    Euler_class.def("e3", &Euler::e3, "float e3()");
    Euler_class.def("Q", &Euler::Q, "float Q()");
    Euler_class.def("n1", &Euler::n1, "float n1()");
    Euler_class.def("n2", &Euler::n2, "float n2()");
    Euler_class.def("n3", &Euler::n3, "float n3()");
    Euler_class.def("Qsgi", &Euler::Qsgi, "float Qsgi()");
    Euler_class.def("n1sgi", &Euler::n1sgi, "float n1sgi()");
    Euler_class.def("n2sgi", &Euler::n2sgi, "float n2sgi()");
    Euler_class.def("n3sgi", &Euler::n3sgi, "float n3sgi()");
    Euler_class.def("setSym", &Euler::setSym,
		    "setSym(string symmetry) \n  initialize symcounter to zero");
    Euler_class.def("Print", &Euler::print,
		    print_overloads("print(string id=None, FILE out=stdout)"));
    
    Euler_class.def("printMatrix", &Euler::printMatrix,
		    Euler_printMatrix("printMatrix(string id=None, FILE out=stdout)"));
    
    Euler_class.def("fromLeft", (void (Euler::*)(Euler)) &Euler::fromLeft,
		    "fromLeft(Euler leftEuler) \n  rotates a given rotation  on the left \n\n fromLeft(alt, az, phi) \n  rotates a given rotation  on the left");

    Euler_class.def("fromLeft", (void (Euler::*)(float, float, float)) &Euler::fromLeft,
		    "fromLeft(Euler leftEuler) \nfromLeft(alt, az, phi) \n  rotates a given rotation  on the left");
    
    Euler_class.def("setAngle", (void (Euler::*)(float, float, float, int))0,
		    Euler_setAngle1("setAngle(float a1, float a2, float a3, EulerType type = Euler.EMAN) \nsetAngle(float e1, float e2, float e3, float e0, EulerType type)"));
		    
    Euler_class.def("setAngle", (void (Euler::*)(float, float, float, float, int))0,
		    "setAngle(float a1, float a2, float a3, EulerType type = Euler.EMAN) \nsetAngle(float e1, float e2, float e3, float e0, EulerType type)");

    Euler_class.def_pickle(Euler_pickle_suite());
}


void add_Quater()
{
    python::class_<Quater> Quater_class("Quater", "Quater()\nQuater(Euler e)\nQuater(float x, float y, float z, float q)");
    Quater_class.def(python::init<>());
    Quater_class.def(python::init<Euler&>());
    Quater_class.def(python::init<float, float, float, float>());
    Quater_class.def("fromEuler", &Quater::fromEuler, "fromEuler(Euler e)");
    Quater_class.def("toEuler", &Quater::toEuler, "Euler toEuler()");
    Quater_class.def("Print", &Quater::print,
		     print_overloads("print(string id = None, FILE out = stdout)"));
    
    Quater_class.def_readwrite("nx", &Quater::nx);
    Quater_class.def_readwrite("ny", &Quater::ny);
    Quater_class.def_readwrite("nz", &Quater::nz);
    Quater_class.def_readwrite("Q", &Quater::Q);
}

void add_Vect()
{
    python::class_<Vect> Vect_class("_Vect", "Don't use this class. Use 'Vect' instead.");
    python::class_<PyVect, python::bases<Vect> > PyVect_class("Vect", "Vect()\nVect(float xx, float yy, float zz)");
    
    PyVect_class.def(python::init<>());
    PyVect_class.def(python::init<float, float, float>());
    
    Vect_class.def("normalize", &Vect::normalize, "normalize()");
    Vect_class.def("rotate",  (Vect (Vect::*)(Euler&)) &Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");

    Vect_class.def("rotate",  (Vect (Vect::*)(Quater&)) &Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");
    
    Vect_class.def("rotate",  (Vect (Vect::*)(Vect&, float)) &Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");

    Vect_class.def("rotate",  (Vect (Vect::*)(float, float, float))&Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");
    
    Vect_class.def("rotate",  (Vect (Vect::*)(float, float, float, float))&Vect::rotate,
		   "Vect rotate(Euler euler)\nVect rotate(float alt, float az, float phi)\nVect rotate(Quater q)\nVect rotate(Vect p, float q)\nVect rotate(float nx, float ny, float nz, float q);");
    
    Vect_class.def("cross", &Vect::cross, "Vect cross(Vect p)");
    Vect_class.def("getQuater", &Vect::getQuater, "Quater getQuater()");
    Vect_class.def("X", &Vect::X, "float X()");
    Vect_class.def("Y", &Vect::Y, "float Y()");
    Vect_class.def("Z", &Vect::Z, "float Z()");
    Vect_class.def("Alt", &Vect::Alt, "float Alt()");
    Vect_class.def("Az", &Vect::Az, "float Az()");
    Vect_class.def("setX", &Vect::setX, "setX(float xx)");
    Vect_class.def("setY", &Vect::setY, "setY(float yy)");
    Vect_class.def("setZ", &Vect::setZ, "setZ(float zz)");
    Vect_class.def("setXYZ", &Vect::setXYZ, "setXYZ(float xx, float yy, float zz)");
    Vect_class.def("inner", &Vect::inner, "float inner(Vect p)");
    Vect_class.def("angle", &Vect::angle,
		   Vect_angle("float angle(Vect p, Vect y=Vect())"));
    
    Vect_class.def("length", &Vect::length, "float length()");
    Vect_class.def("euler", &Vect::euler, "Euler euler()");
    Vect_class.def("eulerTo", (Euler (Vect::*)(Vect&)) &Vect::eulerTo,
		   "Euler eulerTo(Vect p)\nEuler eulerTo(float x, float y, float z)");
    Vect_class.def("eulerTo", (Euler (Vect::*)(float, float, float))&Vect::eulerTo);
    Vect_class.def("isNeighbor", (bool (Vect::*)(Vect&))&Vect::isNeighbor,
		   "bool isNeighbor(Vect p)\n  26 neighbor connection");
    Vect_class.def("Print", &Vect::print, print_overloads("print(string id=None, FILE out=stdout)"));

    
    PyVect_class.def(python::self * float());
    PyVect_class.def(python::self / float());
    PyVect_class.def(python::self + float());
    PyVect_class.def(python::self + python::self);
    PyVect_class.def(python::self - float());
    PyVect_class.def(python::self - python::self);
    PyVect_class.def(python::self == python::self);
}

void add_VolFrag()
{
    python::class_<VolFrag> VolFrag_class("VolFrag", "VolFrag()\n  construct an empty VolFrag instance");
    VolFrag_class.def(python::init<>());

    VolFrag_class.def("setSize", &VolFrag::setSize, 
		      "setSize(int size)\n  sets object size");
    VolFrag_class.def("getSize", &VolFrag::getSize, 
		      "getSize()");
    VolFrag_class.def("setSizeYZ", &VolFrag::setSizeYZ, 
		      "setSizeYZ(int nx, int y, int z)");
    VolFrag_class.def("setSizeI", &VolFrag::setSizeI, 
		      "setSizeI(int nx, int i)");
    VolFrag_class.def("setupIcos", &VolFrag::setupIcos, 
		      "setupIcos()");
    VolFrag_class.def("zero", &VolFrag::zero, 
		      "zero() \n  zero all elements");
    VolFrag_class.def("validXYZ", &VolFrag::validXYZ, 
		      "int validXYZ(int x, int y, int z)");
    VolFrag_class.def("valueAt", &VolFrag::valueAt, 
		      "float valueAt(int x, int y, int z)");
    VolFrag_class.def("setValue", &VolFrag::setValue, 
		      "setValue(int x, int y, int z, float val)");
    VolFrag_class.def("addValue", &VolFrag::addValue, 
		      "addValue(int x, int y, int z, float val)");
    VolFrag_class.def("multValue", &VolFrag::multValue, 
		      "multValue(int x, int y, int z, float val)");
    VolFrag_class.def("addFrag", &VolFrag::addFrag, 
		      "addFrag(VolFrag src)\n  only within the intersection of the 2 frags");
    VolFrag_class.def("writeFile", &VolFrag::writeFile, 
		      "writeFile(filename)\n  return 0 on success not 0 on failure");
    VolFrag_class.def("readFile", &VolFrag::readFile, 
		      "readFile(filename)\n  return 0 on success not 0 on failure");

}

void add_Cylinder()
{
    python::class_<Cylinder> Cylinder_class("Cylinder", "Cylinder()\n construct an empty Cylinder instance\n\nCylinder(Vect center, Vect axisdir, len = 5.4, r = 2.5, w = 0.9)");
    Cylinder_class.def(python::init<>());
    Cylinder_class.def(python::init<const Vect&, const Vect&, float, float, float>());

    Cylinder_class.def("Center", &Cylinder::Center, 
		       "Vect Center()");
    Cylinder_class.def("Axis", &Cylinder::Axis, 
		       "Vect Axis()");
    Cylinder_class.def("QuaterForAxis", &Cylinder::QuaterForAxis, 
		       "Quater QuaterForAxis()");
    Cylinder_class.def("Xc", &Cylinder::Xc, 
		       "float Xc()");
    Cylinder_class.def("Yc", &Cylinder::Yc, 
		       "float Yc()");
    Cylinder_class.def("Zc", &Cylinder::Zc, 
		       "float Zc()");
    Cylinder_class.def("Alt", &Cylinder::Alt, 
		       "float Alt()");
    Cylinder_class.def("Az", &Cylinder::Az, 
		       "float Az()");
    Cylinder_class.def("Radius", &Cylinder::Radius, 
		       "float Radius()");
    Cylinder_class.def("Length", &Cylinder::Length, 
		       "float Length()");
    Cylinder_class.def("Weight", &Cylinder::Weight, 
		       "float Weight()");
    Cylinder_class.def("StartID", &Cylinder::StartID, 
		       "int StartID()");
    Cylinder_class.def("EndID", &Cylinder::EndID, 
		       "int EndID()");
    Cylinder_class.def("Label", &Cylinder::Label, 
		       "string	Label()");
    Cylinder_class.def("Print", &Cylinder::print,
		       print_overloads("Print(int id = -1, FILE out = stdout)"));

    Cylinder_class.def("toInventor", &Cylinder::toInventor,
		       Cylinder_toInventor("toInventor(FILE out = stdout, id = 0, indent = 1, with_head = false)"));
    Cylinder_class.def("toDejaVu", &Cylinder::toDejaVu,
		       Cylinder_toDejaVu("toDejaVu(FILE out = stdout, id = 0)"));

    Cylinder_class.def("toCosec", &Cylinder::toCosec,
		       Cylinder_toCosec("toCosec(FILE out = stdout, id = 0)"));

    Cylinder_class.def("setCenter", &Cylinder::setCenter, 
		       "setCenter(float x, float y, float z)");
    Cylinder_class.def("setAxis", (void (Cylinder::*)(const Vect&)) &Cylinder::setAxis, 
		       "setAxis(Vect axisdir)\nsetAxis(Euler axisdir)");
    Cylinder_class.def("setAxis", (void (Cylinder::*)(Euler&))&Cylinder::setAxis, 
		       "setAxis(Vect axisdir)\nsetAxis(Euler axisdir)");
    Cylinder_class.def("setRadius", &Cylinder::setRadius, 
		       "setRadius(float r)");
    Cylinder_class.def("setLength", &Cylinder::setLength, 
		       "setLength(float l)");
    Cylinder_class.def("setWeight", &Cylinder::setWeight, 
		       "setWeight(float w)");
    Cylinder_class.def("setStartID", &Cylinder::setStartID, 
		       "setStartID(int i)");
    Cylinder_class.def("setEndID", &Cylinder::setEndID, 
		       "setEndID(int i)");
    Cylinder_class.def("setLabel", &Cylinder::setLabel, 
		       "setLabel(string l)");
    Cylinder_class.def("PlaneAngleDiff", &Cylinder::PlaneAngleDiff, 
		       "float PlaneAngleDiff(Cylinder next)\n  dihedral angle of planes formed by the two axes and center connection vector");
    Cylinder_class.def("AxisAngleDiff", &Cylinder::AxisAngleDiff, 
		       "float AxisAngleDiff(Cylinder next)\n  difference in orientation (axises)");
    Cylinder_class.def("Angle2Vect", &Cylinder::Angle2Vect, 
		       "float Angle2Vect(Vect v)\n  difference in orientation axis of this and the direction vector ");
    Cylinder_class.def("Angle2Point", &Cylinder::Angle2Point, 
		       "float Angle2Point(Vect p)\n  difference in orientation between line passing both P and this center and axis of this");
    Cylinder_class.def("Angle2Center", &Cylinder::Angle2Center, 
		       "float Angle2Center(Cylinder next)\n  difference in orientation between line passing both centers and axis of this");
    Cylinder_class.def("NormDistance", &Cylinder::NormDistance, 
		       "float NormDistance(Cylinder next)\n  shortest distance of the two axes ");
    Cylinder_class.def("CenterDistance", &Cylinder::CenterDistance, 
		       "float	CenterDistance(Cylinder next)\n  distance of the center of next cylinder to the center of this");
    Cylinder_class.def("Distance2Point", &Cylinder::Distance2Point, 
		       "float	Distance2Point(x, y, z)\n  distance of the Point to the center of this cylinder");
    Cylinder_class.def("if_merge", &Cylinder::if_merge,
		       Cylinder_if_merge("bool if_merge(Cylinder next, da_limit = 30*PI/180., distance_limit = 3.2)\n  if next is ok to merge with this"));

    Cylinder_class.def("merge", &Cylinder::merge, 
		       "bool merge(Cylinder next)\n  actually merge both into this");
    Cylinder_class.def("mirror", &Cylinder::mirror, 
		       "mirror()\n  mirror the cylinders");
    Cylinder_class.def("Transform", (void (Cylinder::*)(Euler&, Vect&, Vect&))&Cylinder::Transform, 
		       "Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate");
    Cylinder_class.def("Transform", (void (Cylinder::*)(Quater&, Vect&, Vect&))&Cylinder::Transform, 
		       "Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate");


}

void add_Cylinders()
{
    
    python::class_<CylLenComp> CylLenComp_class("CylLenComp", "Cylinder length compare class");
    python::class_<CylWeightComp> CylWeightComp_class("CylWeightComp", "Cylinder weight compare class");
    
    python::class_<Cylinders> Cylinders_class("Cylinders", "A container of Cylinder objects");

    Cylinders_class.def("size", &Cylinders::size, 
			"int size() \n Returns number of Cylinders.");
    Cylinders_class.def("copy", &Cylinders::copy, 
			"Cylinders copy()");
    Cylinders_class.def("subset", &Cylinders::subset, 
			"Cylinders subset(list1, int n)");
    Cylinders_class.def("add", &Cylinders::add, 
			"bool add(Cylinder cyl)");
    
    Cylinders_class.def("ReadListfile", (bool (Cylinders::*)(const char*))0,
			"bool ReadListfile(string in = \"-\");");

    Cylinders_class.def("ReadListfile", (bool (Cylinders::*)(FILE*))0,
			"bool ReadListfile(FILE in = stdin);");

    Cylinders_class.def("WriteListfile", (bool (Cylinders::*)(const char*))0,
			"bool WriteListfile(string out = \"-\");");

    Cylinders_class.def("WriteListfile", (bool (Cylinders::*)(FILE*))0,
			"bool WriteListfile(FILE out = stdout);");

    Cylinders_class.def("ReadInventor", (bool (Cylinders::*)(const char*))0,
			"bool ReadInventor(string in = \"-\");");

    Cylinders_class.def("ReadInventor", (bool (Cylinders::*)(FILE*))0,
			"bool ReadInventor(FILE in = stdin);");

    Cylinders_class.def("WriteInventor", (bool (Cylinders::*)(const char*))0,
			"bool WriteInventor(string out = \"-\");");

    Cylinders_class.def("WriteInventor", (bool (Cylinders::*)(FILE*))0,
			"bool WriteInventor(FILE out = stdout);");

    Cylinders_class.def("WriteVRML", (bool (Cylinders::*)(const char*))0,
			"bool WriteVRML(string out = \"-\");");

    Cylinders_class.def("WriteVRML", (bool (Cylinders::*)(FILE*))0,
			"bool WriteVRML(FILE out = stdout);");

    Cylinders_class.def("ReadDejaVu", (bool (Cylinders::*)(const char*))0,
			"bool ReadDejaVu(string in=\"-\");");

    Cylinders_class.def("ReadDejaVu", (bool (Cylinders::*)(FILE*))0,
			"bool ReadDejaVu(FILE in=stdin);");

    Cylinders_class.def("WriteDejaVu", (bool (Cylinders::*)(FILE*, const char*,const char*, const char*))0,
			Cylinders_WriteDejaVu("bool WriteDejaVu(outfile, mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')\nbool WriteDejaVu(outfilename, mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')"));

    Cylinders_class.def("WriteDejaVu", (bool (Cylinders::*)(const char*, const char*,const char*, const char*))0,
			Cylinders_WriteDejaVu("bool WriteDejaVu(outfile, mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')\nbool WriteDejaVu(outfilename, mol = '0dum', note = 'dummy protein', pdb = '0dum.pdb')"));
    
    Cylinders_class.def("ReadCosec", (bool (Cylinders::*)(const char*))0,
			"bool ReadCosec(string in =\"-\")");

    Cylinders_class.def("ReadCosec", (bool (Cylinders::*)(FILE*))0,
			"bool ReadCosec(FILE in=stdin);");

    Cylinders_class.def("WriteCosec", (bool (Cylinders::*)(FILE*,const char*, const char*, const char*, const char*,const char*, const char*))0,
			Cylinders_WriteCosec("bool WriteCosec(outfile, prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author=''\n\nbool WriteCosec(outfilename, prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author='')"));
    
    Cylinders_class.def("WriteCosec", (bool (Cylinders::*)(const char*,const char*, const char*,const char*, const char*,const char*, const char*))0,
			Cylinders_WriteCosec("bool WriteCosec(outfile, prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author=''\n\nbool WriteCosec(outfilename, prog='helixhunter', pdbid='0dum', header='dummy protein', compnd='dummy protein', source='nowhere', author='')"));
    
    
    Cylinders_class.def("SortLength", &Cylinders::SortLength, 
			"bool SortLength()");
    Cylinders_class.def("SortWeight", &Cylinders::SortWeight, 
			"bool SortWeight()");
    Cylinders_class.def("OrientationDiff", &Cylinders::OrientationDiff,
			Cylinders_OrientationDiff("float OrientationDiff(Cylinders comp, ignoredir=true, uselengthweight=true)\n  average individual cylinders angle diff"));

    Cylinders_class.def("mirror", &Cylinders::mirror, 
			"mirror()");
    Cylinders_class.def("Transform", (void (Cylinders::*)(Quater&, Vect&, Vect&))&Cylinders::Transform, 
			"Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate");
    Cylinders_class.def("Transform", (void (Cylinders::*)(Euler&, Vect&, Vect&))&Cylinders::Transform, 
			"Transform(Quater rot, Vect trans, Vect center)\n  transform: rotate then translate\n\nTransform(Euler rot, Vect trans, Vect center)\n  transform: rotate then translate");
    Cylinders_class.def("Print", &Cylinders::print, "void print()");
    

}

    
void add_ObjectStat()
{
    python::class_<ObjectStat> ObjectStat_class("ObjectStat", "ObjectStat");

    ObjectStat_class.def("SetObject", (void (ObjectStat::*)(EMData*, const std::vector<Vect*>&))&ObjectStat::SetObject, 
			 "SetObject(EMData map, list obj)\n  obj: a list of Vect ojbects\n\nSetObject(EMData map)");
    ObjectStat_class.def("SetObject", (void (ObjectStat::*)(EMData*))&ObjectStat::SetObject, 
			 "SetObject(EMData map, list obj)\n  obj: a list of Vect ojbects\n\nSetObject(EMData map)");
    ObjectStat_class.def("EigenSortTo", &ObjectStat::EigenSortTo, 
			 "EigenSortTo(ObjectStat obj)");
    ObjectStat_class.def("FitEllipsoid", &ObjectStat::FitEllipsoid, 
			 "FitEllipsoid(sort = true)");
    ObjectStat_class.def("Norm", &ObjectStat::Norm, 
			 "Norm()");
    ObjectStat_class.def("ComputeLongAxis", &ObjectStat::ComputeLongAxis, 
			 "ComputeLongAxis()");
    ObjectStat_class.def("Print", &ObjectStat::print, "print()");

}

void add_Helix()
{
    python::class_<Helix> Helix_class("Helix", "definitons for output format for use in IRIS Explorer module");

    Helix_class.def_readwrite("qx", &Helix::qx);
    Helix_class.def_readwrite("qy", &Helix::qy);
    Helix_class.def_readwrite("qz", &Helix::qz);
    Helix_class.def_readwrite("qa", &Helix::qa);
    Helix_class.def_readwrite("tx", &Helix::tx);
    Helix_class.def_readwrite("ty", &Helix::ty);
    Helix_class.def_readwrite("tz", &Helix::tz);
    Helix_class.def_readwrite("c", &Helix::c);
    Helix_class.def_readwrite("len", &Helix::len);
}


void add_util()
{
    python::def("tmpnam2", tmpnam2, "string tmpnam2()");
    python::def("resolv", resolv, "unsigned long resolv(string host = NULL)");
    python::def("sock_open", sock_open, "int sock_open(unsigned long addr, unsigned short port)");
    python::def("writesock", writesock, "int writesock(int sock, string data, int len)");
    python::def("FSsend", FSsend, "int FSsend(int command, int datalen, string filename, string data,int fileno=0)");
    python::def("maxf", (float (*)(float, float)) maxf, "float maxf(float f1, float f2)");
    python::def("maxf", (float (*)(float, float, float)) maxf, "float maxf(float f1,float f2,float f3)");
    python::def("maxf", (float (*)(float, float, float, float)) maxf, "float maxf(float f1,float f2,float f3,float f4)");
    python::def("minf", (float (*)(float, float, float)) minf, "float minf(float f1,float f2,float f3=1.0e30)");
    python::def("minf", (float (*)(float, float, float, float)) minf, "float minf(float f1,float f2,float f3,float f4)");
    python::def("floor2", floor2, "int floor2(float x)");
    python::def("hypot2", hypot2, "float hypot2(float x, float y)");
    python::def("hypot3", hypot3, "float hypot3(float x,float y,float z)");

    python::def("addHTMLSection", addHTMLSection, "void addHTMLSection(string sourcefile, FILE out, string secid)");
    python::def("error", (void (*)(char*, int))error, "void error(string msg, int pri)");
    python::def("error", (void (*)(int, char*))error, "void error(int pri, string msg)");
  
    python::def("file_lock_wait", file_lock_wait, "int file_lock_wait(FILE file)");
    python::def("file_lock", file_lock, "int file_lock(FILE file)");
    python::def("file_unlock", file_unlock, "void file_unlock(FILE file)");

    python::def("LOG", LOG, "void LOG(int ref, string filename, int mode, string text)");
    
    python::def("LOGbegin", py_LOGbegin, "int LOGbegin(list argv, int ppid=0)");
    
    python::def("LOGerror", LOGerror, "void LOGerror(int ref, int err, string text)");
    
    python::def("appinit", py_appinit, "void appinit(list argv) \n  All applications should call this before parsing the command line");
    python::def("appusage", appusage, "void appusage(string argv0)");
    python::def("frand", frand, "float frand(float min, float max)");
    python::def("grand", grand, "float grand(float mean, float sigma)");
    python::def("cyc", cyc, "int cyc(int i, int n)");
    python::def("interpolate", py_interpolate, "float interpolate(float xl, int n, list x_list, list y_list)");
    python::def("bilin", bilin, "float bilin(float p1, float p2, float p3, float p4, float t, float u)");
    
    python::def("p5peak", p5peak, "float p5peak(float v1, float v2, float v3, float v4,float v5) \n this will interpolate a peak location. v3 is considered to be at x=0. The other points are nearest neighbors returns 0 if not well defined");

    python::def("agauss", agauss, "float agauss(float a, float dx, float dy, float dz, float d)");
    python::def("SQR", SQR, "float SQR(float x)");
    python::def("roteulertb", py_roteulertb, "void py_roteulertb(Float alt, Float az, Float phi, float dx, float dy)");
    python::def("npfa", npfa, "int npfa(int low)");
    python::def("isvalidfft", isvalidfft, "int isvalidfft(int low)");
    python::def("bestfftsize", bestfftsize, "int bestfftsize(int low) \n returns a good fft size >= than low");
    
    python::def("save_data", (void (*)(PyList, PyList, int, const char*)) py_save_data, "void py_save_data(list x_list, list y_list, int n, string filename);");
    
    python::def("save_data", (void (*)(float, float, PyList, int, const char*)) py_save_data, "void py_save_data(float x0, float dx, list y_list, int n, string filename);");
    
    python::def("lsqfit", py_lsqfit, "lsqfit(int n, list dx, list dy, Float m, Float b, int ign0 = 1) \n least squares fit returns slope, m, and intercept, b");
    
    python::def("run", run, "int run(string program);");
}

void add_enums()
{
    python::enum_<ImageType>("Image")
	.value("mrc", IMAGE_MRC)
	.value("spider", IMAGE_SPIDER)
	.value("img", IMAGE_IMAGIC)
	.value("pgm", IMAGE_PGM)
	.value("lst", IMAGE_LST)
	.value("pif", IMAGE_PIF)
	.value("dm3", IMAGE_DM3)
	.value("tif", IMAGE_TIFF)
	.value("vtk", IMAGE_VTK)
	.value("sal", IMAGE_SAL)
	.value("map", IMAGE_ICOS)
	.value("img3d", IMAGE_IMAGIC3D)
	.value("emim", IMAGE_EMIM)
	.value("single_spider", IMAGE_SINGLE_SPIDER)
	.value("dm2", IMAGE_GATAN)
	.value("amira", IMAGE_AMIRA)
	.value("xplor", IMAGE_XPLOR)
	.value("any", IMAGE_ANY)
	;

    python::enum_<Euler::EulerType>("EulerType")
	.value("eman", Euler::EMAN)
	.value("imagic", Euler::IMAGIC)
	.value("spin", Euler::SPIN)
	.value("quat", Euler::QUAT)
	.value("matrix", Euler::MATRIX)
	.value("sgirot", Euler::SGIROT)
	.value("spider", Euler::SPIDER)
	.value("mrc", Euler::MRC)
	;
}

BOOST_PYTHON_FUNCTION_OVERLOADS(readImages_overloads, py_readImages, 3, 4);
BOOST_PYTHON_FUNCTION_OVERLOADS(readImagesExt_overloads, py_readImagesExt, 4, 5);

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_copy, py_copy, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_readImage, readImage, 1, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_writeImage, writeImage, 1, 6);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_BilateralFilter, BilateralFilter, 2, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_calcHist, py_calcHist, 2, 5);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_setSize, setSize, 1, 3);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_setRAlign, setRAlign, 1, 3);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_setTAlign, setTAlign, 2, 3);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_calcCCF, py_calcCCF, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_calcCCFX, py_calcCCFX, 1, 4);

BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_calcMCF, py_calcMCF, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_littleBigDot, py_littleBigDot, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_rotAlignCH, rotAlignCH, 1, 3);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_transAlign3d, transAlign3d, 1, 3);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_RTFAlignRadon, py_RTFAlignRadon, 1, 6);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_RTAlignRadon, py_RTAlignRadon, 1, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_RTFAlignSlow, py_RTFAlignSlow, 1, 3);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_RTFAlignSlowest, py_RTFAlignSlowest, 1, 3);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_RTFAlign, py_RTFAlign, 0, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_refineAlign, refineAlign, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_makeRFP, py_makeRFP, 0, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_unwrap, py_unwrap, 0, 6);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_rotateAndTranslate, rotateAndTranslate, 0, 5);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_rotateAndTranslateFast, rotateAndTranslateFast, 0, 1);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_fastTranslate, fastTranslate, 0, 1);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_cmCenter, cmCenter, 0, 1);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_applyMask, applyMask, 2, 5);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_autoMask, autoMask, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_applyRadFn, py_applyRadFn, 4, 5);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_normalizeTo, py_normalizeTo, 1, 3);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_edgeNormalize, edgeNormalize, 0, 1);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_xNormalize, xNormalize, 0, 1);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_leastSquareNormalizeTo, py_leastSquareNormalizeTo, 1, 3);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_maskNormalize, maskNormalize, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_dot, dot, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_lcmp, py_lcmp, 1, 3);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_realFilter, realFilter, 1, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_makeAverage, py_makeAverage, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_makeAverageCTFCWauto, py_makeAverageCTFCWauto, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_makeAverageCTFCW, py_makeAverageCTFCW, 2, 3);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_mult, mult, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_commonLines, commonLines, 2, 5);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_project3d, py_project3d, 4, 5);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_commonLinesR, commonLinesR, 2, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_py_fftSlice, py_fftSlice, 3, 4);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_setup4Slice, setup4Slice, 0, 1);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_insertSlice, insertSlice, 4, 6);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_doneSlice, doneSlice, 0, 1);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_doneSliceWF, doneSliceWF, 0, 1);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_cutSlice, cutSlice, 2, 6);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_uncutSlice, uncutSlice, 2, 5);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_ctfMap, ctfMap, 1, 2);
BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(EMData_dist, dist, 1, 2);



void add_EMData()
{
    python::class_<EMData> EMData_class("_EMData", "Don't use this class. Use EMData instead.");
    EMData_class.def(python::init<>());
    
    python::class_<PyEMData, python::bases<EMData> > PyEMData_class("EMData", "EMData() \n  construct an empty EMData instance");
    PyEMData_class.def(python::init<>());
    
    PyEMData_class.def("copyHead", &PyEMData::py_copyHead,
		       python::return_value_policy<python::manage_new_object>(),
		       "EMData copyHead() \n copies descriptive information without any real data");
    
    PyEMData_class.def("copy", &PyEMData::py_copy,
		       EMData_py_copy("EMData copy(int withfft=0, int withparent=1)\n  returns an exact copy of itself")
		       [python::return_value_policy<python::manage_new_object>()]);

    PyEMData_class.def("clip", (PyEMData* (PyEMData::*)(int, int, int, int)) &PyEMData::py_clip,
		       python::return_value_policy<python::manage_new_object>(),
		       "EMData clip(int x0, int y0, int xsize, int ysize) \nEMData clip(int x0, int y0, int z0, int xsize, int ysize, int zsize) \n  inclusive clip. Pads if larger than data");
    
    PyEMData_class.def("clip", (PyEMData* (PyEMData::*)(int, int, int, int, int, int)) &PyEMData::py_clip,
		       python::return_value_policy<python::manage_new_object>(),
		       "EMData clip(int x0, int y0, int xsize, int ysize) \nEMData clip(int x0, int y0, int z0, int xsize, int ysize, int zsize) \n  inclusive clip. Pads if larger than data");
    
    EMData_class.def("insertClip", &EMData::insertClip, 
		     "insertClip(EMData block, int x0, int y0) \n used for 2D only");
    
    EMData_class.def("readMRCArea", (int (EMData::*)(const char *fsp,int x0,int y0,int w,int h))&EMData::readMRCArea,
		     "int readMRCArea(string fsp, int x0, int y0, int xsize, int ysize) \n  read a rectangular area from a mrc file. works ONLY for real 2d images. Also the area MUST reside within the image. \n\nint readMRCArea(string fsp, int x0, int y0, int z0, int xsize, int ysize, int zsize) \n  read a cube from a 3D mrc file. works ONLY for real 3d images. Also the area MUST reside within the image.");
    
    EMData_class.def("readMRCArea", (int (EMData::*)(const char *fsp,int x0,int y0,int z0,int xl,int yl,int zl))&EMData::readMRCArea,
		     "int readMRCArea(string fsp, int x0, int y0, int xsize, int ysize) \n  read a rectangular area from a mrc file. works ONLY for real 2d images. Also the area MUST reside within the image. \n\nint readMRCArea(string fsp, int x0, int y0, int z0, int xsize, int ysize, int zsize) \n  read a cube from a 3D mrc file. works ONLY for real 3d images. Also the area MUST reside within the image.");
    

    EMData_class.def("readImage", (int (PyEMData::*)(const char*, int, int, ImageType)) &PyEMData::readImage,
		     EMData_readImage(python::args("filename", "n", "nodata", "imagetype"), "int readImage(filename, n = 0, nodata = 0, imagetype = EMData.ANY) \n  Reads a single image from a file. If image type is not specified, the image type will be determined from file type. return 0 if succeeds; return -1 if fails."));
    
    EMData_class.def("writeImage", (int (PyEMData::*)(const char*, int, ImageType, int, int, int)) &PyEMData::writeImage,
		     EMData_writeImage(python::args("filename", "n", "imagetype", "nolock", "swap", "mode"),"int writeImage(filename, n=0, imagetype=EMData.ANY, nolock=0, swap = 0, mode=MODE_float) \n  Writes a single image to a file. If image type is not specified, teh image type will be determined from given file type. return 0 if succeeds; return -1 if fails."));

    python::def("fileCount", (python::tuple (*)(const char*)) fileCount, "tuple fileCount(string imagefile) \n Returns a tuple with the number of images in a file and the image type.");

    python::def("readImages", py_readImages,
		readImages_overloads("list readImages(string imagefile, int n0, int n1, int nodata=0) \n  Read a set of images. return a list of EMData instances ranging from image n0 to image n1. If nodata is 0, only read image header information. If nodata is 1, read both image data and image header."));

    python::def("readImagesExt", py_readImagesExt,
		readImagesExt_overloads("list readImagesExt(string imagefile, string ext, int n0, int n1, int nodata=0) \n  Read a set of filtered images with an automatically generated filename. return a list of EMData instances ranging from image n0 to image n1. If nodata is 0, only read image header information. If nodata is 1, read both image data and image header."));

    
    EMData_class.def("readLST", &EMData::readLST, "int readLST(string filespec, int n, int nodata=0, string file_ext = None) \n  writes a pointer to an image file to a LST file");
    
    EMData_class.def("writeLST", &EMData::writeLST,
		     "int writeLST(string filespec, int n, string reffile, int refn, string comment)");
    
    EMData_class.def("serialIn", &EMData::serialIn, "serialIn(int file_descriptor)");
    
    EMData_class.def("serialOut", &EMData::serialOut, 
		     "serialOut(int file_descriptor) \n  used ONLY for FILESERVER operation, not safe otherwise");
    
    EMData_class.def("writeDebug", &EMData::writeDebug,
		     "writeDebug(int n) \n  write to a special graphical debug viewer");
    
    EMData_class.def("BilateralFilter", (void (EMData::*)(float, float, int, int)) &EMData::BilateralFilter,
		     EMData_BilateralFilter("void BilateralFilter(float sigma1, float sigma2, int iter=1, int localWidth=1);"));
    
    EMData_class.def("calcHist", &PyEMData::py_calcHist,
		     EMData_calcHist("calcHist(list hist, int n, float hmin=0, float hmax=0, int add=0)"));

    EMData_class.def("update", &EMData::update, 
		     "update() \n  Tells EMData to update itself in the future.");
    
    EMData_class.def("setPixel", &EMData::setPixel, 
		     "setPixel(float pix) \n  set the pixel value.");
    EMData_class.def("Pixel", &EMData::Pixel, 
		     "float Pixel() \n  returns the pixel value.");

    EMData_class.def("Min", &EMData::Min,
		     "float Min()");
    EMData_class.def("Max", &EMData::Max,
		     "float Max()");
    PyEMData_class.def("MinLoc", &PyEMData::py_MinLoc, 
		       "tuple MinLoc() \n  Returns minimum value locations in a tuple with (min_x_loc, min_y_loc, min_z_loc)");
    PyEMData_class.def("MaxLoc", &PyEMData::py_MaxLoc, 
		       "tuple MaxLoc() \n  Returns maxium value locations in a tuple with (max_x_loc, max_y_loc, max_z_loc)");

    PyEMData_class.def("display", &PyEMData::display, 
		       "int display() \n  Display the current image. Returns 0 if success; Returns -1 on error.");    
    EMData_class.def("Mean", &EMData::Mean, 
		     "float Mean()");
    EMData_class.def("Sigma", &EMData::Sigma, 
		     "float Sigma()");
    EMData_class.def("SumSq", &EMData::SumSq, 
		     "float SumSq()");
    EMData_class.def("Mean2", &EMData::Mean2, 
		     "float Mean2()");
    EMData_class.def("Sigma2", &EMData::Sigma2, 
		     "float Sigma2()");
    EMData_class.def("Skewness", &EMData::Skewness, 
		     "float Skewness()");
    EMData_class.def("Kurtosis", &EMData::Kurtosis, 
		     "float Kurtosis()");      
    EMData_class.def("edgeMean", &EMData::edgeMean, 
		     "float edgeMean()");
    EMData_class.def("cirMean", &EMData::cirMean, 
		     "float cirMean()");
    EMData_class.def("setParent", &EMData::setParent, 
		     "setParent(EMData data)");
    PyEMData_class.def("Parent", &PyEMData::py_Parent,
		       python::return_internal_reference<>(),
		       "EMData Parent()");
    EMData_class.def("setName", &EMData::setName, 
		     "setName(string name)");
    EMData_class.def("Name", &EMData::Name, 
		     "string Name()");
    EMData_class.def("setPath", &EMData::setPath, 
		     "setPath(string path)");
    EMData_class.def("Path", &EMData::Path, 
		     "string Path()");
    EMData_class.def("setPathN", &EMData::setPathN, 
		     "setPathN(int n)");
    EMData_class.def("PathN", &EMData::PathN, 
		     "int PathN()");

    EMData_class.def("xSize", &EMData::xSize, 
		     "int xSize() \n  Returns x dimension size");
    EMData_class.def("ySize", &EMData::ySize, 
		     "int ySize()  \n  Returns y dimension size");
    EMData_class.def("zSize", &EMData::zSize, 
		     "int zSize() \n  Returns z dimension size");
    EMData_class.def("setSize", &EMData::setSize, 
		     EMData_setSize("int setSize(int x, int y=1, int z=1) \n Sets x, y, z dimension sizes"));
    EMData_class.def("setRI", &EMData::setRI, 
		     "setRI(int i)");
    EMData_class.def("setComplex", &EMData::setComplex, 
		     "setComplex(int i)");
    EMData_class.def("isComplex", &EMData::isComplex, 
		     "int isComplex() \n Returns 1 if the data is in complex number mode; Returns 0 othewise.");
    EMData_class.def("setComplexX", &EMData::setComplexX, 
		     "setComplexX(int i)");
    EMData_class.def("isComplexX", &EMData::isComplexX,
		     "int isComplexX()");
    EMData_class.def("hasCTF", &EMData::hasCTF,
		     "int hasCTF()");
    EMData_class.def("hasCTFF", &EMData::hasCTFF,
		     "int hasCTFF()");
    EMData_class.def("getFlags", &EMData::getFlags, 
		     "int getFlags()");
    EMData_class.def("aliPeak", &EMData::aliPeak, 
		     "float aliPeak()");
    EMData_class.def("setFlipped", &EMData::setFlipped, 
		     "setFlipped(int f)");
    EMData_class.def("isFlipped", &EMData::isFlipped, 
		     "int isFlipped()");
    EMData_class.def("Dx", &EMData::Dx,  
		     "float Dx()");
    EMData_class.def("Dy", &EMData::Dy, 
		     "float Dy()");
    EMData_class.def("Dz", &EMData::Dz, 
		     "float Dz()");
    EMData_class.def("alt", &EMData::alt, 
		     "float alt()");
    EMData_class.def("az", &EMData::az, 
		     "float az()");
    EMData_class.def("phi", &EMData::phi, 
		     "float phi()");
    PyEMData_class.def("getEuler", &PyEMData::py_getEuler,
		       python::return_internal_reference<>(),
		       "Euler getEuler()");

    EMData_class.def("setTAlign", &EMData::setTAlign,
		     EMData_setTAlign("setTAlign(float x, float y, float z=0)"));
    
    EMData_class.def("setRAlign", (void (EMData::*)(Euler*))&EMData::setRAlign,
		     "setRAlign(Euler angle)");
    
    EMData_class.def("setRAlign", (void (EMData::*)(float, float, float))&EMData::setRAlign,
		     EMData_setRAlign("setRAlign(float Alt, float Az=0, float Phi=0)"));

    EMData_class.def("NImg", &EMData::NImg, 
		     "int NImg() \n  Returns the number images in this image file.");
    
    EMData_class.def("getATime", &EMData::getATime, 
		     "long getATime() \n  Returns access time for the image file.");
    
    EMData_class.def("getImagicHed", &EMData::getImagicHed,
		     python::return_internal_reference<>(),
		     "imagicH getImagicHed()");
    EMData_class.def("setImagicHed", &EMData::setImagicHed, 
		     "setImagicHed(imagicH hed)");
    EMData_class.def("getMRCHed", &EMData::getMRCHed,
		     python::return_internal_reference<>(),
		     "mrcH getMRCHed()");
    EMData_class.def("getSpiHed", &EMData::getSpiHed,
		     python::return_internal_reference<>(),
		     "SPIDERH getSpiHed() \n  Returns the header of a spider image.");
    
    EMData_class.def("Busy", &EMData::Busy, "int Busy() \n  Returns 1 if data is checked out");

    EMData_class.def("waitReady", &EMData::waitReady,
		     "int waitReady(int readonly) \n  Waits for data to become available; Returns 0 if OK; Returns non-zero if any error.");
    
    EMData_class.def("Wait", &EMData::Wait, "Wait() \n  Waits until the data becomes available.");
    
    EMData_class.def("swapout", &EMData::swapout, "int swapout() \n  Swaps the data out to a temporary file.");

    EMData_class.def("swapin", &EMData::swapin, "int swapin() \n  Swaps the data in from the swap file.");

    EMData_class.def("zero", &EMData::zero, 
		     "zero() \n  Sets all the data values to be 0.");
    EMData_class.def("one", &EMData::one, 
		     "one() \n  Sets all the data values to be 1.");
    
    EMData_class.def("ri2ap", &EMData::ri2ap, 
		     "ri2ap() \n  converts real/imaginary to amp/pha (complex only)");
    EMData_class.def("ap2ri", &EMData::ap2ri, 
		     "ap2ri() \n  converts amp/pha to real/imaginary (complex only)");
    PyEMData_class.def("doFFT", &PyEMData::py_doFFT,
		       python::return_value_policy<python::manage_new_object>(),
		       "EMData doFFT() \n  Apply fourier transform on the data.");
    
    PyEMData_class.def("doIFT", &PyEMData::py_doIFT,
		       python::return_internal_reference<>(),
		       "EMData doIFT() \n  Apply inverse fourier transform on the data.");
    
    EMData_class.def("gimmeFFT", &EMData::gimmeFFT, 
		     "gimmeFFT() \n  gives the caller ownership of the recently obtained fourier transform, so it won't be freed automatically");
    PyEMData_class.def("doRadon", &PyEMData::py_doRadon,
		       python::return_value_policy<python::manage_new_object>(),
		       " EMData doRadon() \n  Radon transform of image, comes out square");
    PyEMData_class.def("vertACF", &PyEMData::py_vertACF,
		       python::return_value_policy<python::manage_new_object>(),
		       "EMData vertACF(int maxdy) \n  vertical autocorrelation calculated in real-space over a range of translations");
    EMData_class.def("doDWT", &EMData::doDWT, 
		     "doDWT(int basis, int level) \n  in place DWT, basis/level defined elsewhere");
    EMData_class.def("doIWT", &EMData::doIWT, 
		     "doIWT(int basis, int level) \n  inverse wavelet x-form");
    EMData_class.def("DWTFilt", &EMData::DWTFilt, 
		     "DWTFilt(int basis, int level, float thresh) \n  in place nonlinear threshold wavelet filter. thresh is with respect to the 'ideal' threshold");

    PyEMData_class.def("convolute", &PyEMData::py_convolute,
		       python::return_internal_reference<>(),
		       "EMData convolute(EMData with)");
    PyEMData_class.def("calcCCF", &PyEMData::py_calcCCF,
		       EMData_py_calcCCF("EMData calcCCF(EMData with, EMData filter=None) \n  cross correlation between 2 images")
		       [python::return_internal_reference<>()]);
    PyEMData_class.def("calcCCFX", &PyEMData::py_calcCCFX,
		       EMData_py_calcCCFX("EMData calcCCFX(EMData with, int y0=0, int y1=-1, int nosum=0) \n  calculates sum of x-axis CCFs")
		       [python::return_value_policy<python::manage_new_object>()]);
    PyEMData_class.def("calcMCF", &PyEMData::py_calcMCF,
		       EMData_py_calcMCF("EMData calcMCF(EMData with, EMData filter=None) \n  mutual correlation between 2 images")
		       [python::return_internal_reference<>()]);
    
    PyEMData_class.def("littleBigDot", &PyEMData::py_littleBigDot,
		       EMData_py_littleBigDot("EMData littleBigDot(EMData with, int sigma=0) \n  CCF in real-space, designed for cases where with is small")
		       [python::return_internal_reference<>()]);
    
    EMData_class.def("transAlign", &PyEMData::py_transAlign, 
		     "tuple transAlign(EMData with, int useparent, int intonly, int maxshift)  \n  translational alignment using CCF");
    EMData_class.def("rotAlign", &EMData::rotAlign, 
		     "float rotAlign(EMData with) \n  rotational alignment using angular correlation");
    EMData_class.def("rotAlignPrecen", &EMData::rotAlignPrecen, 
		     "float rotAlignPrecen(EMData with) \n  rotational alignment assuming centers are correct");
    EMData_class.def("rotAlignCH", &EMData::rotAlignCH, 
		     EMData_rotAlignCH("float rotAlignCH(EMData with, int irad=8, int orad=0) \n  rotational alignment via circular harmonic"));
    EMData_class.def("transAlign3d", &EMData::transAlign3d, 
		     EMData_transAlign3d("float transAlign3d(EMData with,int useparent=0,int intonly=0) \n  translational alignment using CCF"));
    PyEMData_class.def("RTFAlignRadon", &PyEMData::py_RTFAlignRadon,
		       EMData_py_RTFAlignRadon("EMData RTFAlignRadon(EMData with,int maxshift=-1,EMData thisf=None,EMData radonwith=None ,EMData radonthis=None,EMData radonthisf=None) \n  rotational, translational and flip alignment with Radon transforms")
		       [python::return_internal_reference<>()]);

    PyEMData_class.def("RTAlignRadon", &PyEMData::py_RTAlignRadon,
		       EMData_py_RTAlignRadon("EMData RTAlignRadon(EMData with,int maxshift=-1,EMData radonwith=None,EMData radonthis=None)")
		       [python::return_internal_reference<>()]);
    
    PyEMData_class.def("RTFAlignSlow", &PyEMData::py_RTFAlignSlow,		     
		       EMData_py_RTFAlignSlow("EMData RTFAlignSlow(EMData with,EMData flip=None,int maxshift=-1)\n  rotational, translational and flip alignment using real-space methods, SLOW")
		       [python::return_internal_reference<>()]);
    
    PyEMData_class.def("RTFAlignSlowest", &PyEMData::py_RTFAlignSlowest,		     
		       EMData_py_RTFAlignSlowest("EMData RTFAlignSlowest(EMData with,EMData flip=None,int maxshift=-1)\n  rotational, translational and flip alignment using exhaustive search. VERY SLOW")
		       [python::return_internal_reference<>()]);
    
    PyEMData_class.def("RTFAlign", &PyEMData::py_RTFAlign, 
		       EMData_py_RTFAlign("EMData RTFAlign(EMData with=None,EMData flip=None,int usedot=1,int maxshift=-1) \n  rotational, translational and flip alignment")
		       [python::return_internal_reference<>()]);
    
    PyEMData_class.def("RTAlign", &PyEMData::py_RTAlign,
		       python::return_internal_reference<>(),
		       "EMData RTAlign(EMData with, int usedot=0, int maxshift=0)");
    
    EMData_class.def("refineAlign", &EMData::refineAlign, 
		     EMData_refineAlign("refineAlign(EMData with,int mode=1)"));
    
    PyEMData_class.def("makeRFP", &PyEMData::py_makeRFP,
		       EMData_py_makeRFP("EMData makeRFP(int premasked=0,int unwrap=1)\n  Makes a 'rotational footprint', which is an 'unwound' autocorrelation function. if unwrap is 0 the rfp is not cached. generally the image should be edgenormalized and masked before using this")
		       [python::return_value_policy<python::manage_new_object>()]);
    
    PyEMData_class.def("unwrap", &PyEMData::py_unwrap,
		       EMData_py_unwrap("EMData unwrap(int r1=-1,int r2=-1,int xs=-1,int dx=0,int dy=0,int do360=0)\n  maps polar coordinates to cylindrical coordinates")
		       [python::return_value_policy<python::manage_new_object>()]);
    
    EMData_class.def("calcRCF", &PyEMData::py_calcRCF, 
		     "calcRCF(EMData with, list sum,int NS)");
    
    EMData_class.def("rotateX", &EMData::rotateX, 
		     "rotateX(int dx) \n  This performs a translation of each line along x with wraparound this is equivalent to a rotation when performed on 'unwrapped' maps");
    
    EMData_class.def("rotateAndTranslate", &EMData::rotateAndTranslate, 
		     EMData_rotateAndTranslate("rotateAndTranslate(float scale=1.0,float dxc=0,float dyc=0,float dzc=0,int r=0)"));
    EMData_class.def("rotateAndTranslateFast", &EMData::rotateAndTranslateFast, 
		     EMData_rotateAndTranslateFast("rotateAndTranslateFast(float scale=1.0)\n  rotate and translate using current settings behavior changes if parent is/not set"));

    EMData_class.def("fastTranslate", &EMData::fastTranslate, 
		     EMData_fastTranslate("fastTranslate(int inplace=1)\n  fast integer-only translation"));
    EMData_class.def("rot180", &EMData::rot180, 
		     "rot180()\n  fast rotation by 180 degrees");
    EMData_class.def("cmCenter", &EMData::cmCenter, 
		     EMData_cmCenter("cmCenter(int intonly=1)\n  center at center of mass, ignores old dx,dy"));
    EMData_class.def("applyMask", &EMData::applyMask, 
		     EMData_applyMask("applyMask(int r,int type)\n  applies a circular mask to the data. type=0 is a step cutoff to the mean value type=4 is a step cutoff to zero type=1 fills in with flatband random noise"));
    EMData_class.def("autoMask", &EMData::autoMask, 
		     EMData_autoMask("autoMask(float thresh,float filter=.1) \n  Attempts to automatically mask out the particle, excluding other particles in the box, etc. thresh runs from ~ -2 to 2, negative numbers for dark. protien and positive numbers for light protein (stain) filter is expressed as a fraction of the Fourier radius "));
    EMData_class.def("applyRadFn", &PyEMData::py_applyRadFn, 
		     EMData_applyRadFn("applyRadFn(int n,float x0,float dx, list y,int interp=1)"));
    EMData_class.def("addRadNoise", &PyEMData::py_addRadNoise, 
		     "addRadNoise(int n, float x0, float dx, list y, int interp)");
    EMData_class.def("subtract", &EMData::subtract, 
		     "subtract(EMData data) \n  subtracts 'data' from the current set");
    EMData_class.def("vFlip", &EMData::vFlip, 
		     "vFlip()\n  flips image vertically");
    EMData_class.def("hFlip", &EMData::hFlip, 
		     "hFlip()\n  flips image horizontally");
    EMData_class.def("normalizeTo", &PyEMData::py_normalizeTo, 
		     EMData_normalizeTo("tuple normalizeTo(EMData noisy,int keepzero=0,int invert=0)\n  This will multiply 'this' by a constant so. it is scaled to the signal in 'to'keepzero will exclude zero values, and keep them at zero in the result"));
    EMData_class.def("normalizeMax", &EMData::normalizeMax, 
		     "normalizeMax()\n  mean -> 0, std dev -> 1");
    EMData_class.def("edgeNormalize", &EMData::edgeNormalize, 
		     EMData_edgeNormalize("edgeNormalize(int cir=0)\n  normalizes so mean -> value at edge of image, if cir=1, uses 2 pixel circular border"));
    EMData_class.def("xNormalize", &EMData::xNormalize, 
		     EMData_xNormalize("xNormalize(int mode=0)\n  normalizes each row in the image individually"));
    EMData_class.def("addMaskShell", &EMData::addMaskShell, 
		     "addMaskShell(int shells)\n  Will add additional shells/rings to an existing 1/0 mask image");
    EMData_class.def("leastSquareNormalizeTo", &PyEMData::py_leastSquareNormalizeTo, 
		     EMData_leastSquareNormalizeTo("tuple leastSquareNormalizeTo(EMData to, float low_thresh=FLT_MIN, float high_thresh=FLT_MAX)"));
    EMData_class.def("maskNormalize", &EMData::maskNormalize, 
		     EMData_maskNormalize("maskNormalize(EMData mask,int sigmatoo=1)\n  Uses a 1/0 mask defining a region to use for the zero-normalization, if sigma is 0, std-dev not modified"));
    EMData_class.def("invert", &EMData::invert, 
		     "invert()\n  multiply by -1");
    EMData_class.def("calcRadDist", (void (PyEMData::*)(int, float, float, PyList))&PyEMData::py_calcRadDist, 
		     "calcRadDist(int n,float x0,float dx, list y)");
    EMData_class.def("calcRadDist", (void (PyEMData::*)(int, float, float, PyList, float, float))&PyEMData::py_calcRadDist, 
		     "calcRadDist(int n,float x0,float dx, list y)");

    EMData_class.def("calcAzDist", &PyEMData::py_calcAzDist, 
		     "calcAzDist(int n,float a0,float da, list data,float rmin,float rmax)");
    EMData_class.def("radialAverage", &EMData::radialAverage, 
		     "radialAverage()\n  makes image circularly symmetric");
    EMData_class.def("radialSubtract", &EMData::radialSubtract, 
		     "radialSubtract()\n  subtracts circularly symmetric part of image");
    EMData_class.def("toCorner", &EMData::toCorner,
		     "toCorner \n Translates a centered image to the corner");

    EMData_class.def("dot", &EMData::dot, 
		     EMData_dot("float dot(EMData data,int evenonly=0)\n  dot product of 2 images (same size !)"));
    EMData_class.def("lcmp", &PyEMData::py_lcmp, 
		     EMData_lcmp("tuple lcmp(EMData noisy,int keepzero=0,int invert=0)\n  linear comparison of 2 data sets (smaller better)"));
    EMData_class.def("pcmp"  , &EMData::pcmp,
		     "float pcmp(EMData data, XYData snr=None) \n  amplitude weighted mean phase error, 'data' should be less noisy than 'this' pcmp");
    EMData_class.def("fscmp", &EMData::fscmp, 
		     " float fscmp(EMData with)\n  returns a quality factor based on FSC between images larger numbers indicate a better match");
    
    EMData_class.def("dCenter", &EMData::dCenter, 
		     "float dCenter()\n  This will find the density value at the peak of the image histogram, sort of like the mode of the density");
    EMData_class.def("sigDiff", &EMData::sigDiff, 
		     "float sigDiff()\n  Calculates sigma above and below the mean and returns the difference between them");
    EMData_class.def("realFilter", &EMData::realFilter, 
		     EMData_realFilter("realFilter(int type, float val1=0,float val2=0, float val3=0)\n  a variety of real-space filters"));
    EMData_class.def("filter", &EMData::filter, 
		     "filter(float highpass,float lowpass,int type)\n  fourier filters an image, real or imaginary");
    EMData_class.def("makeAverage", &PyEMData::py_makeAverage, 
		     EMData_makeAverage("makeAverage(list in,EMData sigma=None)\n  averages the images in the list, result in 'this' optionally makes a sigma image as well"));
    EMData_class.def("makeAverageIter", &PyEMData::py_makeAverageIter, 
		     "makeAverageIter(list in)");
    EMData_class.def("makeAverageWT", &PyEMData::py_makeAverageWT, 
		     "makeAverageWT(list in,EMData curves,XYData sf)\n  averages the images in the list with SNR weightingbut no CTF correction");
    EMData_class.def("makeAverageCTFC", &PyEMData::py_makeAverageCTFC, 
		     "makeAverageCTFC(list in,float filtr)\n  averages the images in the list with CTF correction filtr is in angstroms,");
    EMData_class.def("makeAverageCTFCWauto", &PyEMData::py_makeAverageCTFCWauto, 
		     EMData_makeAverageCTFCWauto("makeAverageCTFCWauto(list in, string outfile=None)\n  This does CTF correction with a Wiener filter where the filter is estimated directly from the data"));
    EMData_class.def("makeAverageCTFCW", &PyEMData::py_makeAverageCTFCW, 
		     EMData_makeAverageCTFCW("makeAverageCTFCW(list in, XYData sf, list SNR=None)\n  averages the images in the list with CTF correction"));
    EMData_class.def("makeMedian", &PyEMData::py_makeMedian, 
		     "makeMedian(list in)\n  Median of the images in the list, result in 'this'");

    EMData_class.def("add", (void (EMData::*)(EMData*))&EMData::add, 
		     "add(EMData in)\n  adds 'in' to 'this'");

    EMData_class.def("add", (void (EMData::*)(float))&EMData::add, 
		     "add(float d)\n  add const number to each element");

    EMData_class.def("addIncoherent", &EMData::addIncoherent, 
		     "addIncoherent(EMData in)\n  adds 'in' to 'this'");
    EMData_class.def("mult", &EMData::mult, 
		     EMData_mult("mult(EMData in,int recip=0)\n  multiplies 'in' by 'this', if recip, then divide"));
    EMData_class.def("multConst", &EMData::multConst, 
		     "multConst(float)\n  multiply by constant");
    EMData_class.def("addConst", &EMData::addConst, 
		     "addConst(float)\n add a constant");
    EMData_class.def("meanShrink", &EMData::meanShrink, 
		     " meanShrink(int i)\n  reduces the size of the image by a factor of i, using the average value of the pixels in a block");
    EMData_class.def("medianShrink", &EMData::medianShrink, 
		     "medianShrink(int i)\n  reduces the size of the image by a factor of i using a local median filter");
    EMData_class.def("commonLines", &EMData::commonLines, 
		     EMData_commonLines("commonLines(EMData d1,EMData d2,int mode=0,int steps=180,int horiz=0) \n  finds common lines between 2 complex images. mode 0 is a summed dot-product mode 1 is weighted phase residualsteps is 1/2 the resolution of the )map"));
    EMData_class.def("commonLinesR", &EMData::commonLinesR, 
		     EMData_commonLinesR("commonLinesR(EMData data1,EMData data2,int steps=180,int horiz=0)"));
    PyEMData_class.def("project3d", &PyEMData::py_project3d,		     
		       EMData_py_project3d("EMData project3d(float alt,float az, float phi, int mode, float thr=1.0) \n  makes a 3d projection of a volume, mode=-1 does real space projection, 1-5 use fftslice for fourier projection")
		       [python::return_value_policy<python::manage_new_object>()]);
    
    PyEMData_class.def("fftSlice", &PyEMData::py_fftSlice, 
		       EMData_py_fftSlice("EMData fftSlice(float alt,float az,float phi,int mode=5)\n  takes a slice from a 3d complex image for generating projections.")
		       [python::return_value_policy<python::manage_new_object>()]);
    
    EMData_class.def("setup4Slice", &EMData::setup4Slice, 
		     EMData_setup4Slice("setup4Slice(int redo=1)"));
    
    EMData_class.def("interpFT3d", &PyEMData::py_interpFT3d, 
		     "interpFT3d(float x,float y,float z, list ret,int mode)");
    
    EMData_class.def("setup4IS", &EMData::setup4IS, 
		     " setup4IS(int size)");
    EMData_class.def("normSlice", &PyEMData::py_normSlice, 
		     "tuple normSlice(EMData slice,float alt,float az,float phi)\n  try to make sure the slice is normalized properly");
    EMData_class.def("insertSlice", &EMData::insertSlice, 
		     EMData_insertSlice("insertSlice(EMData slice,  float alt,float az,float phi,int mode=5,float weight=1.0) \n  this inserts a complex slice into an image that has been setup4IS()"));
    EMData_class.def("insertSliceWF", &EMData::insertSliceWF, 
		     "insertSliceWF(EMData slice,  float alt,float az,float phi,int mode,list SNR,float padratio)\n  this inserts a complex slice into an image that has been setup4IS()");
    EMData_class.def("doneSlice", &EMData::doneSlice, 
		     EMData_doneSlice("doneSlice(int log=0)\n   once all of the slices have been added with insertSlice(), this routine does a normalization and returns a real 3d volume"));
    EMData_class.def("doneSliceWF", &EMData::doneSliceWF, 
		     EMData_doneSliceWF("doneSliceWF(int log=0)\n  It should only be used in conjunction with insertSliceWF"));
    EMData_class.def("insertSlice3DReal", &EMData::insertSlice3DReal, 
		     "insertSlice3DReal(EMData slice,float weight)");

    EMData_class.def("cutSlice", (void (EMData::*)(EMData*, float, Euler*, int, float, float))&EMData::cutSlice,
		     EMData_cutSlice("void cutSlice(EMData *map,float z,Euler *ort=NULL,int interp=1,float x=0,float y=0);\n  this will cut a slice out of a real 3D map"));

    EMData_class.def("uncutSlice", (void (EMData::*)(EMData*, float, Euler*, float, float))&EMData::uncutSlice,
		     EMData_uncutSlice("void uncutSlice(EMData *map,float z,Euler *ort=NULL,float x=0,float y=0);\n  Opposite of cutSlice(). It will take a slice and insert the data into a real 3D map. It does not interpolate, it uses the nearest neighbor."));
    
    
    EMData_class.def("ctfMap", &EMData::ctfMap, 
		     EMData_ctfMap("ctfMap(int type, XYData sf=None)"));
    EMData_class.def("getCTF", &PyEMData::py_getCTF,
		     "list getCTF() \n  Returns the CTF parameters in a list.");

    EMData_class.def("setCTF", &PyEMData::py_setCTF, 
		     "setCTF(list new_ctf) \n  Sets CTF parameters with new values from a list. \n\nsetCTFF(string s) \n  Sets CTF parameters with new values from a string.");

    EMData_class.def("setCTFF", &EMData::setCTFF, 
		     "setCTF(list new_ctf) \n  Sets CTF parameters with new values from a list. \n\nsetCTFF(string s) \n  Sets CTF parameters with new values from a string.");
    
    EMData_class.def("clearCTF", &EMData::clearCTF, 
		     "clearCTF() \n  Clear CTF parameters.");
    
    EMData_class.def("CTFCmp", &EMData::CTFCmp, 
		     "int CTFCmp(EMData with)\n  used to detect if 2 images have the same CTF parameters");

    EMData_class.def("subNoise", &EMData::subNoise, 
		     "subNoise() \n  Subtract average noise from intensity. Negative results allowed.");

    PyEMData_class.def("iftSlice", &PyEMData::py_iftSlice,
		       python::return_internal_reference<>(),
		       "EMData iftSlice()\n  This does an IFT of a volume reconstructed from slices. Includes a real-space correction for the linear interpolation in Fourier space");

    EMData_class.def("valueAt", (float (EMData::*)(int, int, int))&EMData::valueAt, 
		     "float valueAt(int x, int y) \n  Return data at a 2D position. \n\nfloat valueAt(int x, int y, int z) \n  Return data at a 3D position.");
    
    EMData_class.def("valueAt", (float (EMData::*)(int, int))&EMData::valueAt, 
		     "float valueAt(int x, int y) \n  Return a data value at a 2D position. It has no locking. \n\nfloat valueAt(int x, int y, int z) \n  Return a data value at a 3D position. It has no locking.");

    EMData_class.def("valueAtInterp", &EMData::valueAtInterp,
		     "float valueAtInterp(float x, float y) \n Returns a data value at a 2D position.");
    
    EMData_class.def("valueAtSafe", (float (EMData::*)(int, int, int))&EMData::valueAtSafe, 
		     "float valueAtSafe(int x, int y) \n  Return a data value at a 2D position. It has no locking. \n\nfloat valueAtSafe(int x, int y, int z) \n  Return a data value at a 3D position. It has no locking.");
    
    EMData_class.def("valueAtSafe", (float (EMData::*)(int, int))&EMData::valueAtSafe, 
		     "float valueAtSafe(int x, int y) \n  Return a data value at a 2D position. It has no locking. \n\nfloat valueAtSafe(int x, int y, int z) \n  Return a data value at a 3D position. It has no locking.");
    
    EMData_class.def("setValueAt", &EMData::setValueAt, 
		     "setValueAt(int x, int y, int z, float v)\n it does not set the changed flag");
    PyEMData_class.def("row", &PyEMData::py_row,
		       python::return_value_policy<python::manage_new_object>(),
		       "EMData row(int n)\n  extract vector from matrix");
    PyEMData_class.def("col", &PyEMData::py_col,
		       python::return_value_policy<python::manage_new_object>(),
		       "EMData col(int n)\n  extract vector from matrix");
    EMData_class.def("setRow", &EMData::setRow, 
		     "setRow(EMData d,int n)\n  insert vector into matrix");
    EMData_class.def("setCol", &EMData::setCol, 
		     "setCol(EMData d,int n)\n  insert vector into matrix");
    PyEMData_class.def("mxMult", &PyEMData::py_mxMult,
		       python::return_internal_reference<>(),
		       " EMData mxMult(EMData second)");
    EMData_class.def("dist", &EMData::dist, 
		     EMData_dist("float dist(EMData second, int n=0)"));

    EMData_class.def_pickle(EMData_pickle_suite());

    PyEMData_class.def(python::self += float());
    PyEMData_class.def(python::self -= float());
    PyEMData_class.def(python::self *= float());
    PyEMData_class.def(python::self /= float());

    PyEMData_class.def(python::self += python::self);
    PyEMData_class.def(python::self -= python::self);
    PyEMData_class.def(python::self *= python::self);
    PyEMData_class.def(python::self /= python::self);
    
    PyEMData_class.def(python::self + float());
    PyEMData_class.def(python::self - float());
    PyEMData_class.def(python::self * float());
    PyEMData_class.def(python::self / float());

    PyEMData_class.def(float() + python::self);
    PyEMData_class.def(float() - python::self);
    PyEMData_class.def(float() * python::self);
    PyEMData_class.def(float() / python::self);

    PyEMData_class.def(python::self + python::self);
    PyEMData_class.def(python::self - python::self);
    PyEMData_class.def(python::self * python::self);
    PyEMData_class.def(python::self / python::self);
}


BOOST_PYTHON_MODULE(libpyEM)
{
    python::scope().attr("__doc__") = "EMAN classes and routines for image/volume processing in single particle reconstructions.\n\nThis exports:\n  - EMData: the primary class to process EM data.\n  - Euler: used to store the orientation of a slice. \n  - Quater  \n  - Vect \n  - VolFrag \n  - Cylinder \n  - Cylinders \n  - ObjectStat \n  - Helix\n";
    
    add_Euler();
    add_Quater();
    add_Vect();
    add_VolFrag();
    add_Cylinder();
    add_Cylinders();
    add_ObjectStat();
    add_Helix();
    add_util();
    add_enums();
    add_EMData();
}
