// align3dsym.C - Copyright 06/15/98 Steve Ludtke
// Copyright 1998 Baylor College of Medicine

//N align3dsym
//F This will rotate a 3D model so its symmetric axes are aligned on the predefined axes
//T LA
//1
//P <input file>	Source file
//P <output file>	Destination file
//P sym=<Cn,Dn>	Apply an n-fold rotational symmetry to each image
//P [mask=<rad>]	Outside mask 
//P [imask=<rad>]	Inner mask applied only for location purposes. Not applied to output.
//P [refine]	Used when the orientation is off by less than 10 degrees
//U align3dsym threed.0a.mrc threed.0a.mrc sym=c7
//D This program will rotate 3D model so it's rotational symmetries are aligned on
//E the predefined axes. The model MUST be centered properly before this program
//E is used. By default the n-fold symmetric axis is z, and a 2-fold axis (for D symmetry)
//E lies along x. Note that this program does not currently work on icosahedral particles.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

inline float asub(float x,float y) {
float r;

r=fabs(x-y);
if (r>PI) r=2.0*PI-r;

return r;
}

main(int argc, char *argv[]) {
int i,Ref,csym,nx,dsym=0,nca=0,imask=-1,mask=-1;
EMData *d1,*d2,*m,*p,*p2;
float alt,az,phi,q,r,best,eul[3],eul2[3],beul[3];
FILE *p3d=NULL;
int refine=0; 

appinit(argc,argv);

if (argc<4) appusage(argv[0]);


if (strncmp(argv[3],"sym=",4)!=0) {
	printf("Please specify sym\n");
	exit(0);
}

if (tolower(argv[3][4])=='c') {
	csym=atoi(&argv[3][5]);
}
else if (tolower(argv[3][4])=='d') {
	csym=atoi(&argv[3][5]);
	dsym=1;
}
else { printf("Printf only c and d symmetries currently supported\n"); exit(1); }

if (argc>4 && strncmp(argv[4],"nocalign",8)==0) nca=1;

for (i=4; i<argc; i++) {
	if (strncmp(argv[4],"nocalign",8)==0) nca=1;
	else if (strncmp("imask=",argv[i],6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp("mask=",argv[i],5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp("refine",argv[i],6)==0) refine=1;
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}


printf("Read start image\n");
d1=new EMData;
d1->readImage(argv[1],-1);
d1->edgeNormalize();
d1->applyMask(mask,4);
if (imask>0) d1->applyMask(imask,5);

//d1->cmCenter(0);
d1->normalize();
nx=d1->xSize();
//d1=d1->clip(-nx/4,-nx/4,-nx/4,nx+nx/2,nx+nx/2,nx+nx/2);
//d1->toCorner();
d1=d1->copy();

m=new EMData;
m->setSize(d1->xSize(),d1->ySize(),1);
m->one();
m->applyMask(nx/2,4);

Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);

best=-1.0e15;
beul[0]=beul[1]=beul[2]=0;
phi=0;

float sr=15.0;
if (refine) sr=1.0;

if (!nca) {
for (r=sr; r>=.05; r/=6.0) {
	for (alt=beul[0]-r*6.0; alt<=beul[0]+r*6.0; alt+=r) {
		for (az=beul[1]-r*6.0; az<=beul[1]+r*6.0; az+=r) {
			if (r/6.0<.05) p=d1->project3d(alt*PI/180.0,az*PI/180.0,phi*PI/180.0,-1);
			else p=d1->project3d(alt*PI/180.0,az*PI/180.0,phi*PI/180.0,-2);
			p->mult(m);
			p2=p->copy();
			q=0;
			for (i=1; i<csym; i++) {
				p2->setRAlign(i*PI*2.0/csym,0,0);
				p2->rotateAndTranslate();
				q+=4.0-p2->lcmp(p);
			}
			delete p2;
			delete p;
			printf("\r%1.2f,%1.2f  ",alt,az);
			fflush(stdout);
			if (q>best) { 
				eul[0]=alt; eul[1]=az; eul[2]=phi; best=q; 
				printf("b-> %1.2f,%1.2f   %1.3g    ",alt,az,best);
			}
		}
	}
	beul[0]=eul[0];
	beul[1]=eul[1];
	beul[2]=eul[2];
}
}
printf("\n\nBest is %1.2f,%1.2f\n",beul[0],beul[1]);

if (dsym) {
	d1->setRAlign(beul[0]*PI/180.0,beul[1]*PI/180.0,beul[2]*PI/180.0);
	d1->rotateAndTranslate();
	d2=d1->copy();
	
	best=-1.0e20;
	beul[2]=0;
	for (r=360.0/(csym*8.0); r>.04; r/=8.0) {
		for (phi=beul[2]-r*8.0; phi<=beul[2]+r*8.0; phi+=r) {
			d2->setRAlign(PI,0,phi*PI/180.0);
			d2->rotateAndTranslate();
			q=d2->dot(d1);
			printf(" Euler %1.1f\t",phi);
			if (q>best) { best=q; eul[2]=phi; printf("Best phi -> %1.1f  %1.2g\n",phi,q);  }
			else printf("\r");
			fflush(stdout);
		}
		beul[2]=eul[2];
	}

	d2->setRAlign(PI,0,beul[2]*PI/180.0);
	d2->rotateAndTranslate();
	d2->add(d1);
	d2->writeMRC("sym.mrc");

	beul[2]/=-2.0;
}

d1->readImage(argv[1],-1);
d1->edgeNormalize();
d1->applyMask(mask,4);
//d1->cmCenter(0);
d1->setRAlign(beul[0]*PI/180.0,beul[1]*PI/180.0,beul[2]*PI/180.0);
d1->rotateAndTranslate();
d1->writeMRC(argv[2]);
LOG(Ref,argv[2],LOG_OUTFILE,NULL);

LOG(Ref,NULL,LOG_END,NULL);
}
