// classalign2 ?/98  Steve Ludtke
// Copyright 1998 Baylor College of Medicine 

//N classalign2
//F Aligns a set of images iteratively to generate a class-average.
//T LA
//1
//P <input file>	Input file, usually a cls*img file
//P <iterations>	Number of iterative loops
//P [ref1]	Indicates that the first image in the file is a reference
//P [ign1]	Indicates that the first image is a dummy which should be ignored
//P [finalref]	Once the average has been generated, a final alignment to the reference is performed
//P [saveali]	Saves the individual aligned images in a separate file
//P [savecen]	Save the translationally (but not rotationally) aligned images
//P [savesteps]	Save the class average after each iterative step to monitor convergence
//P [badfile]	Save all excluded images in 'bad' files.
//P [stat]	Writes individual image statistics to 'class.stat'
//P [saverefs]	Write the reference image as well as the class average to the output file
//P [mask=<rad>]	Circular mask radius for the class average
//P [refmask]	This will use a mask generated from the reference image. Assumes the 3D model has been sensibly masked.
//P [maxshift=<rad>]	Maximum translation allowed in alignment. If not specified, implied value from mask is used.
//P [keep=<sigmamult>]	The threshold value for keeping images. Standard deviation multiplier	
//P [locfromfile]	Determine the output file image position from the input filename.
//P [median]	When CTF correction is NOT used, this produces better statistics.
//P [ctfc=<res>]	Enables CTF correction, thru resolution <res> in angstroms.
//P [ctfcw=<SF file>]	Enables CTF correction, with Wiener filtration, requires s vs inten struct. factor file.
//P [ctfcw]	Enables CTF correction, with Wiener filtration, SNR is estimated from the data (DOESN'T WORK)
//P [fixac=<0 to 1>]	This allows you to set the %AC value in the image headers to a fixed value. Note that this will NOT compensate defocus for change in %AC.
//P [imask=<rad>]	Inside mask radius.
//P [sigfilt]	Filters the class averages so only intense values are used in alignment
//P [even|odd]	Only use the even or odd images in the input file
//P [sigmaimg]	Generate a standard deviation image
//P [even/odd]	Processes only the even or odd numbered images in the file (used by ttest)
//P [logit=<label>]	This specifies a label to use in the particle.log file. This records particle number classification throughout a refinement.
//P [refine]	This enables 1/2 pixel accuracy in alignment, it is slow, and rarely worthwhile.
//P [usefilt]	This will use a filtered dataset for alignment, but uses the real data for averaging.
//P	[slow]	This will do alignment via exhaustive search. It is VERY slow.
//U classalign2 cls000.hed 8 ref1 finalref saverefs mask=32 keep=.8 locfromfile ctfc=15
//D This is one of the most important programs in EMAN. It is responsible for taking a set
//E of particles within a single class, that is, they are supposed to be in the same 
//E orientation, and performing a mutual alignement to generate a class-average. This 
//E program is largely responsible for the model independance of EMAN, ie - why EMAN can 
//E converge in so few iterations. This program is also where CTF amplitude correction occurs. 

//D There are quite a few options for this program. It is typically called from classalignall,
//E which has almost identical options. The few options that are present here, but not in 
//E classalignall, are intended for occasional use when checking robustness of the proceedures. 

//D There are 3 manditory options which are specified in all cases. First, is the
//E &lt;input file>. This file contains the images which are to be used to generate a class
//E average. Optionally, the first image in this file may be a reference image with
//E assigned euler angles. The reference will be used only during the first iteration
//E of the alignment process, so the final class average is nearly independent of this
//E reference. It also provides a final rotational orientation for the class average. 

//D The number of iterations (with no prefixed text) and the
//E 'keep=&lt;sigmamult>' values must also be specified. This program functions
//E by: first, aligning all of the images to a common reference, then
//E averaging the aligned images together to generate a new reference. This
//E reference is then used for another round of alignment, etc. The number
//E of iterations specifies how many times this process should repeat.
//E Each time, the images should generate a class average more consitent
//E with the data. In addition, after each loop, each aligned image is
//E compared with the class average from the previous iteration. If the
//E quality of the match is worse than a certain threshold value, the image
//E is discarded. Each iteration this threshold is made more stringent
//E until &lt;sigmamult> is reached in the final iteration. The cutoff value
//E is determined by calculating the standard deviation of all of the match
//E qualities and multiplying by &lt;sigmamult>. The smaller &lt;sigmamult> is,
//E the more consistent the images in the final class average will be, but
//E as more images are excluded, the class average will also become more
//E noisy. Similarly, increasing the number of iterations will increase the
//E consistency of the final class average, but will also typically cause
//E more data to be discarded. There are no 'correct' values to use. There
//E is always a tradeoff between data homogeneity and noise level.  Typical
//E starting values are 8 iterations with &lt;sigmamult>=1.2. &lt;sigmamult> is
//E usually >= 1.0, but may be lower when data homogenity is very poor. More
//E iterations can always be used, but 8 is usually sufficient to converge
//E quite well.

//D When the program runs, it displays some useful information on the
//E screen. First you will see 1 or more rows of '.'s appear. Each '.'
//E represents an image being aligned. If a '*' appears instead of a '.',
//E that means that particular image was below the threshold and discarded.
//E Note that '*' images are only excluded from the average during a
//E particular cycle. They may be included again in later cycles if the
//E class average 'wanders' in their direction.

//D If your input file contains a reference image (which the other images
//E are expected to look at least vaguely similar to), then 'ref1' should be
//E specified. Usually 'finalref' is also specified. Normally the initial
//E reference is used only during the first iteration, however, during
//E several iterations the orientation of the class average in the plane
//E can drift. 'finalref' insures that the final class average is oriented
//E as closely as possible to the orientation of the initial reference. It
//E does not change the appearance of the class average, only its orientation.

//D 'saverefs' is also usually specified with 'ref1'. This causes the
//E reference images to be saved along with the class averages in the output
//E file.  This is used to compare each reference with the corresponding class
//E average. When the refinement cycle begins to converge, the reference
//E images should be nearly identical to the class averages, although the
//E class averages will always be noisier.

//D 'saveali', 'savesteps', 'badfile' and 'stat' are all used primarily when
//E something seems to be going wrong. They cause various output files to be
//E generated, which can then be used to see exactly what's happening. They
//E are rarely used in a normal refinement cycle.

//D The default output image from classalign2 is a class average. This means,
//E each pixel is equal to the average of that pixel's value in each of the
//E aligned images. However, very often the median value will have better
//E properties than the mean value (less affected by outliers). Median
//E images can be generated instead of averages by specifying the 'median'
//E option. Note that this option is incompatible with CTF correction (see
//E 'ctfc' below).

//D Usually, classalign2 will simply append each output class average to the
//E end of 'classes.hed'. However, if the program is being run in parallel,
//E it is usually desirable to preserve the ordering of the images in the
//E output file. 'locfromfile' will cause the output images to be put in
//E specific 'slots' in the output file, overwriting any image already in that
//E location in the file. This is used by default when classalignall is run.

//D 'sigfilt' is used when a refinement is failing to produce consistent
//E results in the early rounds of refinement. This applies a non-linear
//E noise reducing filter to the reference image to try to get better
//E initial alignment. It is generally only used when necessary, and then,
//E only for early refinement rounds. It should not be used when the final
//E few refinement iterations are performed.

//D 'ctfc' and 'ctfcw' are provided to enable CTF correction when
//E generating class averages. For this to work properly all of the images
//E in the input file must be pretreated by the ctfit program, so the
//E phases are flipped and the CTF parameters are stored in the individual
//E image headers. See the documentation for that program 
//E and <a href=../ctfc/ctfc.html> the CTF correction primer</a>
//E for a discussion of how CTF correction
//E is performed. ctfc is specified with a filter resolution for the final
//E image. Otherwise high resolution noise will dominate. eg - 'ctfc=12' will
//E apply a 12A filter to the final image. NOTE: This IS specified in Angstroms,
//E not pixels like most other filter options.
//D 'ctfcw' is the preferred variant, but requires an x-ray scattering curve
//E or a simulated x-ray scattering curve. This curve MUST be used directly
//E with ctfit when the data is pretreated, or the class average will be 
//E improperly filtered. This option applies a true Wiener filter to the
//E data based on the SNR (signal to noise ratio) estimated from the data
//E combined with the scattering curve. eg- 'ctfcw=groel.sm' would ctf-correct
//E the data with an applied wiener filter. groel.sm would be the 2-column
//E data file defining the x-ray scattering curve.

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include "XYData.h"

main(int argc, char *argv[]) {
int i,j,k,n,ny,iter=2,r1=0,savesteps=0,badf=0,stat=0,ign1=0;
int refine=0,saver=0,half=0,sigfilt=0,savecen=0,filtali=0,slow=0;
int mask=-1,imask=-1,locffile=-1,median=0,saveali=0,ctfc=0,precen=0,quiet=0,maxshift=-1,radon=0;
List *data,*dataf,*drad,*dradf,*d2a,*d2ma,*dataa=NULL;
EMData *d,*da,*df,*d2,*avg,*oavg=NULL,*first=NULL,*firstf=NULL,*mask1,*masknorm=NULL,*tmp,*sigim=NULL;
EMData *reforig=NULL;
char s[1024],*ctfcw=NULL,*logit=NULL;
float f,m,mean,sigma,mean2,sigma2,cut,keep=1.2,norm,filt=0,fixac=-1.0;
float eul[3];
float *ctf;
FILE *out;
XYData SF;

appinit(argc,argv);

if (argc<2) { appusage(argv[0]); exit(0); }

for (i=2; i<argc; i++) {
	if (strncmp("ref1",argv[i],4)==0) r1=1;
	else if (strncmp("ign1",argv[i],4)==0) ign1=1;
	else if (strncmp("finalref",argv[i],8)==0) r1=2;
	else if (strncmp("savesteps",argv[i],9)==0) savesteps=1;
	else if (strncmp("badfile",argv[i],7)==0) badf=1;
	else if (strncmp("sigfilt",argv[i],7)==0) sigfilt=1;
	else if (strncmp("sigmaimg",argv[i],8)==0) sigim=new EMData;
	else if (strncmp("saverefs",argv[i],8)==0) saver=1;
	else if (strncmp("saveali",argv[i],7)==0) saveali=1;
	else if (strncmp("savecen",argv[i],7)==0) savecen=1;
	else if (strncmp("precen",argv[i],6)==0) precen=1;
	else if (strncmp("radon",argv[i],5)==0) radon=1;
	else if (strncmp("slow",argv[i],4)==0) slow=1;
	else if (strncmp("stat",argv[i],4)==0) stat=1;
	else if (strncmp("ctfc=",argv[i],5)==0) { ctfc=1; filt=atof(&argv[i][5]); }
	else if (strncmp("ctfcw=",argv[i],6)==0) { ctfc=2; ctfcw=&argv[i][6]; }
	else if (strncmp("ctfcw",argv[i],5)==0) { ctfc=2; }
	else if (strncmp("fixac=",argv[i],6)==0) fixac=atof(&argv[i][6]);
	else if (strncmp("mask=",argv[i],5)==0&&mask!=-2) mask=atoi(&argv[i][5]);
	else if (strncmp("maxshift=",argv[i],9)==0) maxshift=atoi(&argv[i][9]);
	else if (strncmp("refmask",argv[i],7)==0) mask=-2;
	else if (strncmp("imask=",argv[i],6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp("keep=",argv[i],5)==0) keep=atof(&argv[i][5]);
	else if (strncmp("locfromfile",argv[i],11)==0) locffile=1;
	else if (strncmp("median",argv[i],6)==0) median=1;
	else if (strncmp("refine",argv[i],6)==0) refine=1;
	else if (strncmp("even",argv[i],4)==0) half=2;
	else if (strncmp("odd",argv[i],3)==0) half=1;
	else if (strncmp("usefilt",argv[i],7)==0) filtali=1;
	else if (strncmp(argv[i],"logit=",6)==0) logit=&argv[i][6];
	else if (strncmp("quiet",argv[i],5)==0) quiet=1;
	else if (isdigit(argv[i][0])) iter=atoi(argv[i])-2;
	else if (strncmp(argv[i],"usefs=",6)==0) ;
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}
if (r1==1) iter++;
if (iter<1) exit(1);
if (ctfc) median=0;
if (mask==-2 && !r1) error(ERR_EXIT,"'refmask' requires 'ref1'. I need a reference to generate the mask from!");

//if (imask>0 && !precen) { printf("The imask option must be used in conjunction with precen\n"); exit(0); }

// This reads the structure factor file used for Wiener filtration, and takes the required log10()
if (ctfc==2) {
	if (ctfcw) {
		if (ctfcw && SF.readFile(ctfcw)) { printf("Couldn't read structure factor file!\n"); exit(1); }
	
		float *d=SF.getData();
		for (i=1; i<SF.n()*2; i+=2) d[i]=(d[i]<=0)?-4.0:log10(d[i]);
		SF.update();
	}
}

printf("Classalign: %s\n",argv[1]);

n=fileCount(argv[1]);
//if (half) n/=2;
if ((n<5 && !r1) || (n<2 && r1) || (n<3 && half)) { printf("Not enough elements\n"); exit(0); }
if (n<10) iter=1;	// only do 2 passes if <10 data sets to average
if (n<3) iter=0;	// only 1 pass if a single image

if (filtali) {
	if (r1) {
		EMData *ed=new EMData;
		ed->readImage(argv[1],0);
		data=readImagesExt(argv[1],".filt",1,n);
		data->insertObjectAt(ed,0);
	}
	else data=readImagesExt(argv[1],".filt",0,n);
	dataa=readImages(argv[1],0,n);
}
else data=readImages(argv[1],0,n);		// read all input images
if (ign1) data->removeObjectAt(0);

if (data==NULL) error("Cannot read input",ERR_EXIT);
printf("Read %d images.\n",data->count());

if (half) {
	n=data->count();
	if (r1) {
		if (half==1) {
			for (i=n-1; i>=1; i-=2) data->removeObjectAt(i);
			if (dataa) { for (i=n-1; i>=1; i-=2) dataa->removeObjectAt(i); }
		}
		else {
			for (i=n-2; i>=1; i-=2) data->removeObjectAt(i);
			if (dataa) {for (i=n-2; i>=1; i-=2) dataa->removeObjectAt(i); }
		}
	}
	else {
		if (half==1) {
			for (i=n-1; i>=0; i-=2) data->removeObjectAt(i);
			if (dataa) { for (i=n-1; i>=0; i-=2) dataa->removeObjectAt(i); }
		}
		else {
			for (i=n-2; i>=0; i-=2) data->removeObjectAt(i);
			if (dataa) { for (i=n-2; i>=0; i-=2) dataa->removeObjectAt(i); }
		}	
	}
	n=data->count();
	printf("Processing %d images\n",n);
}

int Ref=0;
if (!quiet) {
	Ref=LOGbegin(argc,argv);
	LOG(Ref,argv[1],LOG_INFILE,NULL);
}

if (mask==-2) {
	mask1=((EMData *)data->objectAt(0))->copy(0,0);
	mask1->realFilter(6);	// value != 0 -> 1
	masknorm=mask1->copy(0,0);
	masknorm->addMaskShell(1);
	masknorm->subtract(mask1);
	masknorm->addMaskShell(2);
//mask1->writeIMAGIC("mask.hed",0);
//masknorm->writeIMAGIC("mask.hed",1);
}
else {
	mask1=((EMData *)data->objectAt(0))->copyHead();
	mask1->one();
	mask1->applyMask(mask,4);
}
//mask1->writeMRC("mask.img");

d2a=new List;
d2ma=new List;

dataf=new List;
for (i=0; i<n; i++) {
	d=(EMData *)data->objectAt(i);
	if (fixac>=0 && d->hasCTF() && ctfc) d->getCTF()[3]=fixac;
//	d->normalize();
	d->edgeNormalize(1);	// normalize to circular edgemean
//if (i==0) d->writeIMAGIC("abc.hed",-1);
//	d->applyMask(-1,0);
	d2=d->copy();
	d2->hFlip();
	d2->setParent(NULL);
	dataf->addObject(d2);

	if (dataa) {
		d=(EMData *)dataa->objectAt(i);
		d->edgeNormalize();
	}	
	
//	d=d->copy();
}

if (radon) {
	printf("Precalculate Radon transforms\n");
	drad=new List;
	dradf=new List;
	
	for (i=0; i<n; i++) {
		if (i%10==0) { printf(" %8d        \r",i); fflush(stdout); }
		d=(EMData *)data->objectAt(i);
		drad-> addObject(d->doRadon());
		d=(EMData *)dataf->objectAt(i);
		dradf->addObject(d->doRadon());
	}
}

d=(EMData *)data->objectAt(0);
ny=d->ySize();
if (maxshift<0) {
	if (mask<=0) maxshift=ny/8; else maxshift=ny/2-mask;
}

if (sigim) { sigim->setSize(ny,ny,1); sigim->zero(); }

// If we're starting with a reference, we don't need the first pass
if (r1) {
	avg=d=(EMData *)data->objectAt(0);
	first=avg->copy();
	first->setParent(NULL);
	data->removeObjectAt(0);
	dataf->removeObjectAt(0);
	if (dataa) dataa->removeObjectAt(0);
	if (mask<0) {
		norm=d->xSize()*d->ySize()*PI/4000.0;
	}
	else{
		norm=mask*mask*PI/1000.0;
	}
	eul[0]=d->alt();  
	eul[1]=d->az();  
	eul[2]=d->phi();
	printf("Eulers %f,%f,%f\n",eul[0]*180.0/PI,eul[1]*180.0/PI,eul[2]*180.0/PI);
	n--;
}

// simple shell sort, fast enough
d=(EMData *)data->objectAt(0);
if ((d->hasCTF()||d->hasCTFF()) && ctfc && (!filtali)) {
	for (i=0; i<n-1; i++) {
		d=(EMData *)data->objectAt(i);
		for (j=i+1; j<n; j++) {
			d2=(EMData *)data->objectAt(j);
			if (d->CTFCmp(d2)<0) {
				data->swap(i,j);
				dataf->swap(i,j);
				if (dataa) dataa->swap(i,j);
				d=(EMData *)data->objectAt(i);
			}
		}
	}
}

//if (r1!=1)
if (!r1) {
	// pass 1, align to the first data set
	d=(EMData *)data->objectAt(0);
	if (mask<0) {
		norm=d->xSize()*d->ySize()*PI/4000.0;
	}
	else{
		norm=mask*mask*PI/1000.0;
	}
	avg=d->copy();
	for (i=1; i<n; i++) {
		d=(EMData *)data->objectAt(i);
		df=(EMData *)dataf->objectAt(i);
		if (precen) d2=d->RFAlign(avg,df);
		else if (radon) d2=d->RTFAlignRadon(avg,maxshift,df,NULL,(EMData *)drad->objectAt(i),(EMData *)dradf->objectAt(i));
		else if (slow) d2=d->RTFAlignSlow(avg,df,maxshift);
		else d2=d->RTFAlign(avg,df,1,maxshift);
//		d2->mult(mask1);
		d2->applyMask(mask,6);
		avg->add(d2);
		delete d2;
		putchar('o'); fflush(stdout);
	}
	putchar('\n');
	if (savesteps) avg->writeIMAGIC("steps.hed",-1);
	if (!precen) {
		avg->cmCenter();
		if (savesteps) avg->writeIMAGIC("steps.hed",-1);
	}
}

#ifdef DEBUG
printf("DEBUG 1\n");
#endif

if (n==1) iter=0;
cut=0;
for (j=0; j<iter; j++) {
// pass 2-?, align to the first average
	mean=sigma=m=0;
	if (reforig && reforig!=oavg) delete reforig;
	if (oavg) { delete oavg; oavg=NULL; }
	reforig=avg;
//	oavg->writeIMAGIC("aaa.hed",-1);
	for (i=0; i<n; i++) {
		d=(EMData *)data->objectAt(i);		// data to align
		df=(EMData *)dataf->objectAt(i);	// flipped version of data

		// if there's a CTF, modify the reference by the same CTF
		if((d->hasCTF()||d->hasCTFF()) && ctfc && (!filtali)) {
			// if several images in a row have the same ctf, don't recalculate
			if (!oavg || d->CTFCmp(oavg)) {
				if (oavg) delete oavg;
				float *ctfp=d->getCTF();
				float B=ctfp[1];		// temporarily sets B factor to 0, doesn't work for CTFF
				ctfp[1]=0;
				ctf=d->ctfCurve(2);
				d2=reforig->doFFT()->copy();
				d2->applyRadFn(CTFOS*d->ySize()/2,0,1.0/CTFOS,ctf,0);
				ctfp[1]=B;
				oavg=d2->doIFT();
				d2->gimmeFFT();
				if (masknorm) oavg->maskNormalize(masknorm);
				else oavg->edgeNormalize();
//oavg->writeMRC("tst1.mrc");
				oavg->mult(mask1);
				oavg->setParent(NULL);
				if (d->hasCTFF() && ctfc) oavg->setCTFF(d->Name());
				else oavg->setCTF(d->getCTF());
				delete d2;
				free(ctf);
//oavg->writeIMAGIC("aaa.hed",-1);
			}
		}
		else {
			if (masknorm) reforig->maskNormalize(masknorm);
			else reforig->edgeNormalize();
			reforig->mult(mask1);
			oavg=reforig;
		}

		if (sigfilt) oavg->realFilter(1,oavg->Sigma());
		
		if (precen) d2=d->RFAlign(oavg,df);
		else {
			if (radon) d2=d->RTFAlignRadon(oavg,maxshift,df,NULL,(EMData *)drad->objectAt(i),(EMData *)dradf->objectAt(i));
			else if (slow) d2=d->RTFAlignSlow(oavg,df,maxshift);
			else d2=d->RTFAlign(oavg,df,1,maxshift);
			if (refine && (radon || j>iter-4)) d2->refineAlign(oavg,1);
		}
		if (d->hasCTF() && ctfc) d2->setCTF(d->getCTF());
		if (d->hasCTFF() && ctfc) d2->setCTFF(d->Name());
//		d2->mult(mask1);
//		d2->applyMask(mask,6);
//		f=oavg->dot(d2);
		float scale=1.0,shift=0;
#ifndef ALTCMP
		f=250.0*(5.0-d2->lcmp(oavg,1,1,&scale,&shift));
#else
		d2->lcmp(oavg,1,1,&scale,&shift);	// sigh, need to do this better, but we need scale & shift
		f=20.*d2->fscmp(oavg);
#endif
		d2->ufloat=f;
		d2->multConst(scale);	// we must normalize each particle to the average before averageCTFCW
		d2->addConst(shift);
//printf("%f %f %f\n",oavg->lcmp(d2),cut,f);
//d2->writeIMAGIC("aaa.hed",-1);
//printf(" %f,%f\n",f,cut);

		d2ma->addObject(d2);
		mean+=f;
		sigma+=f*f;

		putchar('?'); fflush(stdout);


/*		if ((f>cut || keep>5 || (i>n-3 && m<2)) && finite(f)) {
			d2a->addObject(d2);
			putchar('.'); fflush(stdout);
			mean+=f;
			sigma+=f*f;
			m++;
		} else {
			putchar('*'); fflush(stdout);	
			delete d2;
		}*/
	}
	putchar('\r');

// calculate statistics
	mean/=(float)n;
	sigma=sqrt(sigma/(float)n-mean*mean);
	if (!finite(mean)) mean=1.0;
	if (!finite(sigma)) sigma=mean/4.0;
	if (sigma==0) exit(0);		// down to 1 image, just quit
	cut=mean-sigma*keep*(((float)j/(iter-2.0))+1.8*(iter-j-1.0)/(iter-2.0));

	mean2=sigma2=m=0;
// decide which images to keep
	for (i=0; i<n; i++) {
		d2=(EMData *)d2ma->objectAt(i);
//		f=oavg->dot(d2);
//		f=250.0*(5.0-d2->lcmp(oavg,1,1));
		f=d2->ufloat;
		
		if ((f>cut || keep>5 || (i>n-3 && m<2)) && finite(f)) {
			putchar('.'); fflush(stdout);
			if (dataa) {
				da=((EMData *)dataa->objectAt(i))->copy(0,0);
				if (d2->isFlipped()) da->hFlip();
				da->setRAlign(d2->alt());
				da->setTAlign(d2->Dx(),d2->Dy(),0);
//printf("%f,%f,%f\n",d2->alt(),d2->Dx(),d2->Dy());
				da->rotateAndTranslate();
				if (da->hasCTFF() && ctfc) da->setCTFF(d2->Name());
				else da->setCTF(d2->getCTF());
				delete d2;
				d2=da;
			}
			d2->setTAlign(0,0);
			d2->setRAlign(eul[0],eul[1],eul[2]);
//			d2->mult(mask1);
			d2a->addObject(d2);
			mean2+=f;
			sigma2+=f*f;
			m++;
		} else {
			putchar('*'); fflush(stdout);		
			delete d2;
		}
	}
	putchar('\n');
	d2ma->empty();

	mean2/=(float)m;
	sigma2=sqrt(sigma2/(float)m-mean2*mean2);
	if (!finite(mean2)) mean2=1.0;
	if (!finite(sigma2)) sigma2=0;

	if (savesteps) oavg->writeIMAGIC("steps.hed",-1);

	avg=new EMData;
	if (median) avg->makeMedian(d2a); 
	else if (ctfc==1) avg->makeAverageCTFC(d2a,filt);
	else if (ctfc==2) {
		if (ctfcw) avg->makeAverageCTFCW(d2a,&SF);
		else avg->makeAverageCTFCWauto(d2a,"snr0.dat");
	}
	else avg->makeAverage(d2a);
//	avg->edgeNormalize();
	if (masknorm) avg->maskNormalize(masknorm);
	else {
		avg->multConst(1.0/avg->Sigma());
		avg->addConst(-avg->dCenter());
	}
	
//avg->writeMRC("tst2.mrc");
	avg->mult(mask1);
//	avg->applyMask(mask,6);
	
	for (i=0; i<d2a->count(); i++) { d2=(EMData *)d2a->objectAt(i); delete d2; }
	d2a->empty();
	

//	printf("Mean=%f\tSigma=%f\tCut=%f\twt=%f\n",mean/norm,sigma/norm,cut/norm,
//		(((float)j/(iter-2.0))+1.8*(iter-j-1.0)/(iter-2.0)));
//	printf("Mean=%f\tSigma=%f\n",mean2/norm,sigma2/norm);

	printf("Mean=%f\tSigma=%f\tCut=%f\twt=%f\n",mean2/norm,sigma2/norm,cut/norm,
		(((float)j/(iter-2.0))+1.8*(iter-j-1.0)/(iter-2.0)));

	// if we have an original reference, align the early averages to it
	// just to get the orientation correct
	if (avg->Sigma()==0 || !finite(avg->Sigma())) printf("Bad Sum\n");

//	if (j<iter-2 && r1) {
	if (r1) {
		if (oavg) { delete oavg; reforig=NULL; }
		df=avg->copy();
		df->setParent(NULL);
		df->hFlip();
		if (precen) oavg=d->RFAlign(first,df);
		else if (radon) oavg=avg->RTFAlignRadon(first,maxshift);
		else if (slow) oavg=avg->RTFAlignSlow(first,df,maxshift);
		else oavg=avg->RTFAlign(first,df,1,maxshift);
//		printf("firstalign %p %5.2f %5.2f %5.2f\n",oavg,oavg->Dx(),oavg->Dy(),oavg->Daz()*180.0/PI);
		if (oavg) { 
			delete avg;
			delete df;
			oavg->setParent(NULL);
			avg=oavg;
			oavg=NULL;
			avg->setRAlign(0,0,0);
			avg->setTAlign(0,0,0);
			avg->update();
		}
	}
	if (avg->Sigma()==0 || !finite(avg->Sigma())) printf("Bad Sum\n");
	if (savesteps) avg->writeIMAGIC("steps.hed",-1);
	if (!r1 && !precen) {
		avg->cmCenter();		// if we have an initial reference
								// don't spoil the centering
		if (savesteps) avg->writeIMAGIC("steps.hed",-1);
	}
}

if (savesteps && !quiet) LOG(Ref,"steps.hed",LOG_OUTFILE,NULL);
#ifdef DEBUG
printf("DEBUG 2\n");
#endif


// last pass, align to the final previous average
mean=sigma=m=0;
//oavg=avg;
//oavg->normalize();
if (reforig && reforig!=oavg) delete reforig;
if (oavg) { delete oavg; oavg=NULL; }
reforig=avg;
//reforig->normalize();

if (stat) out=fopen("class.stat","a");
#ifdef DEBUG
printf("DEBUG 2a\n");
#endif

for (i=0; i<n; i++) {
	d=(EMData *)data->objectAt(i);
	df=(EMData *)dataf->objectAt(i);

	// if there's a CTF, modify the reference by the same CTF
	if ((d->hasCTF()||d->hasCTFF()) && ctfc && (!filtali)) {
		// if several images in a row have the same ctf, don't recalculate
		if (!oavg || d->CTFCmp(oavg)) {
			if (oavg) delete oavg;
			float *ctfp=d->getCTF();
			float B=ctfp[1];		// temporarily sets B factor to 0, doesn't work for CTFF
			ctfp[1]=0;
			ctf=d->ctfCurve(2);
			d2=reforig->doFFT()->copy();
			d2->applyRadFn(CTFOS*d->ySize()/2,0,1.0/CTFOS,ctf,0);
			ctfp[1]=B;
			oavg=d2->doIFT();
			d2->gimmeFFT();
			if (masknorm) oavg->maskNormalize(masknorm);
			else oavg->edgeNormalize();
			oavg->mult(mask1);
			oavg->setParent(NULL);
			if (d->hasCTFF()) oavg->setCTFF(d->Name());
			else oavg->setCTF(d->getCTF());
			delete d2;
			free(ctf);
		}
	}
	else {
		if (masknorm) reforig->maskNormalize(masknorm);
		else reforig->edgeNormalize();
		reforig->mult(mask1);
		oavg=reforig;
	}
//oavg->writeIMAGIC("aaa.hed",-1);

#ifdef DEBUG
printf("DEBUG 2b\n");
#endif
	if (sigfilt) oavg->realFilter(1,oavg->Sigma());

	if (precen) d2=d->RFAlign(oavg,df);
	else if (slow) d2=d->RTFAlignSlow(oavg,df,maxshift);
	else {
		d2=d->RTFAlign(oavg,df,1,maxshift);
		if (refine) d2->refineAlign(oavg,1);
	}
	if (d->hasCTF() && ctfc) d2->setCTF(d->getCTF());
	if (d->hasCTFF() && ctfc) d2->setCTFF(d->Name());
//	d2->mult(mask1);
//	d2->applyMask(mask,6);
//d2->writeIMAGIC("aaa.hed",-1);
//	f=oavg->dot(d2);
	float scale=1.0,shift=0;
#ifndef ALTCMP
	f=250.0*(5.0-d2->lcmp(oavg,1,1,&scale,&shift));
#else
	d2->lcmp(oavg,1,1,&scale,&shift);	// sigh, need to do this better, but we need scale & shift
	f=20.*d2->fscmp(oavg);
#endif

	d2->ufloat=f;
	d2->multConst(scale); // we must normalize each particle to the average before averageCTFCW
	d2->addConst(shift);

	if (sigim) {
		tmp=d2->copy(0);
		tmp->normalizeTo(oavg);
		tmp->subtract(oavg);
		tmp->mult(tmp);
		sigim->add(tmp);
		delete tmp;
	}

	d2->setNImg(1);
//	d2->setRAlign(eul[0],eul[1],eul[2]);
	d2ma->addObject(d2);
	mean+=f;
	sigma+=f*f;
	putchar('-'); fflush(stdout);

/*	if ((f>cut || keep>5) &&finite(f)) {
		d2->setNImg(1);
		d2->setRAlign(eul[0],eul[1],eul[2]);
		d2a->addObject(d2);
		mean+=f;
		sigma+=f*f;
		m++;
	}
	else {
		sprintf(s,"b%s",argv[1]);
		if (badf) d->writeIMAGIC(s,-1);
		delete d2;
	} 

	if (stat) fprintf(out,"%f\n",f/norm);
	printf("%6.1f %c  ",f/norm,f>cut?' ':'*');
	if (i%8==0) putchar('\n');
	fflush(stdout); */
#ifdef DEBUG
printf("DEBUG 2e\n");
#endif
}
putchar('\n');

if (sigim) {
	sigim->multConst(1.0/(float)n);
	sigim->realFilter(10,0,0,0);	// sqrt
	sigim->setRAlign(eul[0],eul[1],eul[2]);
	sigim->setNImg((int)n);
	sigim->writeIMAGIC("classsig.hed",-1);
}

#ifdef DEBUG
printf("DEBUG 3\n");
#endif

mean/=(float)n;
sigma=sqrt(sigma/(float)n-mean*mean);
if (!finite(mean)) mean=1.0;
if (!finite(sigma)) sigma=mean/4.0;
if (sigma==0) exit(0);		// down to 1 image, just quit
cut=mean-sigma*keep*(((float)j/(iter-2.0))+1.8*(iter-j-1.0)/(iter-2.0));

mean2=sigma2=m=0;

// decide which images to keep
for (i=0; i<n; i++) {
	d2=(EMData *)d2ma->objectAt(i);
//	f=oavg->dot(d2);
//	f=250.0*(5.0-d2->lcmp(oavg,1,1));
	f=d2->ufloat;

	if (stat) fprintf(out,"%f\n",f/norm);
	printf("%6.1f %c  ",f/norm,f>cut?' ':'*');
	if (i%8==0) putchar('\n');
	fflush(stdout);

	if ((f<=cut || keep>5) && logit) {
		char s[120];
		sprintf(s,"X\t%d\t%s\n",d2->PathN(),logit);
		FSsend(FS_TEXT,strlen(s)+1,"particle.log",s);
	}
	
	if ((f>cut || keep>5) &&finite(f)) {
		if (dataa) {
			da=((EMData *)dataa->objectAt(i))->copy(0,0);
			if (d2->isFlipped()) da->hFlip();
			da->setRAlign(d2->alt());
			da->setTAlign(d2->Dx(),d2->Dy(),0);
//printf("%f,%f,%f\n",d2->alt(),d2->Dx(),d2->Dy());
			da->rotateAndTranslate();
			if (d2->hasCTFF() && ctfc) da->setCTFF(d2->Name());
			else da->setCTF(d2->getCTF());
			delete d2;
			d2=da;
		}
		d2->setTAlign(0,0);
		d2->setRAlign(eul[0],eul[1],eul[2]);
//		d2->mult(mask1);
		d2a->addObject(d2);			
		mean2+=f;
		sigma2+=f*f;
		m++;
	} else {
		sprintf(s,"b%s",argv[1]);
		if (badf) d2->writeIMAGIC(s,-1);
		delete d2;
	}
}
putchar('\n');
d2ma->empty();

#ifdef DEBUG
printf("DEBUG 4\n");
#endif

mean2/=(float)m;
sigma2=sqrt(sigma2/(float)m-mean2*mean2);
if (!finite(mean2)) mean2=1.0;
if (!finite(sigma2)) sigma2=0;


//if (badf && !quiet) LOG(Ref,"nogood.hed",LOG_OUTFILE,NULL);

EMData SNR;
SNR.setSize(CTFOS*ny/2,1,1);
if (d2a->count()==0) { printf("ERROR: No images left!\n"); exit(0); }
avg=new EMData;
if (median) avg->makeMedian(d2a); 
else if (ctfc==1) avg->makeAverageCTFC(d2a,filt);
else if (ctfc==2) {
	if (ctfcw) { avg->makeAverageCTFCW(d2a,&SF,SNR.getData()); SNR.doneData(); }
	else avg->makeAverageCTFCWauto(d2a,"snr.dat");
}
else avg->makeAverage(d2a);

sprintf(s,"a%s",argv[1]);
if (saveali && !quiet) LOG(Ref,s,LOG_OUTFILE,NULL);
if (savecen && !quiet) LOG(Ref,"start.cen.hed",LOG_OUTFILE,NULL);
for (i=0; i<d2a->count(); i++) { 
	d2=(EMData *)d2a->objectAt(i);
	if (saveali) {
		d2->writeIMAGIC(s,-1);
	}
	if (savecen) {
//		printf("%4.1f\t%4.1f\n",d2->Dx(),d2->Dy());
		d2->setRAlign(0,0,0);
		d2->setTAlign(floor(d2->Dx()+.5),floor(d2->Dy()+.5),0);
		d2->rotateAndTranslate();
		d2->writeIMAGIC("start.cen.hed",-1);
	}
	delete d2; 
}
d2a->empty();

#ifdef DEBUG
printf("DEBUG 5\n");
#endif

//if (!r1) avg->cmCenter();
if (stat) fclose(out);

//mean/=(float)n;
//sigma=sqrt(sigma/(float)n-mean*mean);
printf("Mean=%f\tSigma=%f\n%0.0f/%d good images\n",mean/norm,sigma/norm,m,data->count());
printf("Mean=%f\tSigma=%f\n",mean2/norm,sigma2/norm);

if (locffile!=-1) {
	locffile=atoi(argv[1]+strlen(argv[1])-8);
	if (saver) locffile*=2;
}


// Final alignment to reference
/*
if (r1==2) {
	avg->RTAlign(first);
	f=avg->alt();
//	f=f-PI*floor(f/PI+.5);
	printf("Final realign %4.2f  (%4.2f %4.2f %4.2f)\n",f*180.0/PI,eul[0]*180.0/PI,eul[1]*180.0/PI,eul[2]*180.0/PI);
//	eul[2]-=f; 		// CORRECTED 2/4/01
	eul[2]+=f;
}*/

if (r1==2) {
	EMData *a2=avg->copy();
	float a,dt,b=0,bd=-1.0e30,bb=0,bm=1.0;
	for (a=-4.0*PI/180.0; a<4.1*PI/180.0; a+=.5*PI/180.0) {
		a2->setRAlign(a);
		a2->rotateAndTranslate();
//		dt=a2->dot(first);
#ifndef ALTCMP
		dt=250.0*(5.0-a2->lcmp(first,1,1));
#else
		dt=20.*a2->fscmp(first);
#endif

		if (dt>bd) { bd=dt; b=a; a2->normalizeTo(first,1,1,&bm,&bb); }
	}
	printf("Final realign %4.2f  (%4.2f %4.2f %4.2f)\n",b*180.0/PI,eul[0]*180.0/PI,eul[1]*180.0/PI,eul[2]*180.0/PI);
	eul[2]+=b;
//	mask1->setRAlign(-b);
//	mask1->setTAlign(0,0);
//	mask1->rotateAndTranslate();
	avg->multConst(bm);
//	avg->addConst(bb);
}
else avg->multConst(1.0/avg->Sigma());

if (saver) {
	first->setNImg(-1);
	first->normalize();
	first->writeIMAGIC("classes.hed",locffile);
	if (locffile>=0) locffile++;
}

if (badf && r1) {
	sprintf(s,"b%s",argv[1]);
	first->writeIMAGIC(s,0);
}

#ifdef DEBUG
printf("DEBUG 6\n");
#endif

//avg->edgeNormalize();
//
if (masknorm) avg->maskNormalize(masknorm,0);
else avg->addConst(-avg->dCenter());
avg->mult(mask1);
//avg->applyMask(mask,6);
avg->setRAlign(eul[0],eul[1],eul[2]);
avg->setNImg((int)m);
avg->writeIMAGIC("classes.hed",locffile);
if (ctfc==2 && ctfcw) SNR.writeIMAGIC("SNR.hed",locffile);
if (!quiet) LOG(Ref,"classes.hed",LOG_OUTFILE,NULL);
if (savesteps) avg->writeIMAGIC("steps.hed",-1);
if (!quiet) LOG(Ref,NULL,LOG_END,NULL);
}


