#!/bin/env python

import os
import sys
import random
import time
import string
import EMAN
from math import *
from sys import argv

ERR_EXIT=1
ERR_CRIT=2
ERR_WARN=3
ERR_INFO=4
def error(type,msg):
	if (type==ERR_CRIT) : print "Critical Error: ",msg
	if (type==ERR_WARN) : print "Warning: ",msg
	if (type==ERR_INFO) : print msg
	if (type==ERR_EXIT) :
		print "Fatal Error: ",msg
		sys.exit(1)

def d2r(x):
	"degrees to radians"
	return x*3.14159/180.0

def coords(i,s):
	x=i%(s*S)
	z=int(floor(i/(s*s)))
	y=i-z*s*s

# MAIN
if (len(argv)<2) :
	print "fastfoldhunter.py <map to search> <search map or pdb> [apix=<apix>]"
	sys.exit(1)

apix=1.0

for i in argv[3:] :
	s=string.split(i,'=')
	if (s[0]=="apix") : apix=float(s[1])
	else: error(ERR_EXIT,"Unknown argument "+i)

# read the imare we're searching
target=EMAN.EMData()
target.readImage(argv[1],-1)

# read the probe image
probe=EMAN.EMData()
probe.readImage(argv[2],-1)
shrink=int(2**floor(log(probe.xSize())/log(2.0)-3))

fsize=int(floor(2.0*target.xSize()/shrink))
if (fsize%4!=0) : fsize-=fsize%4

# first stage, shrink a LOT
targets=target.copy(0,0)
targets.meanShrink(shrink)

print "Using an initial shrink of %d giving a target size of %d padded to %d"%(shrink,targets.xSize(),fsize)

txs=(fsize-targets.xSize())/2
targets.edgeNormalize()
targets=targets.clip(-txs,-txs,-txs,fsize,fsize,fsize)

pxs=int((fsize-floor(probe.xSize()/shrink))/2)

# probe must be same size as image
#probes.edgeNormalize()
#probes=probe.copy(0,0)
#probes.meanShrink(shrink)
#probes=probes.clip(-txs,-txs,-txs,fsize,fsize,fsize)

ang=[(0,0),(45,0),(45,90),(45,180),(45,270),(90,0),(90,60),(90,120),(90,180),(90,240),(90,300),(135,0),(135,90),(135,180),(135,270),(180,0)]

n=0
loclist=[]
for aa in ang:
	for phi in range(0,320,45):
		prober=probe.copy(0,0)
		prober.setRAlign(d2r(aa[0]),d2r(aa[1]),d2r(phi))
		prober.rotateAndTranslate()

		prober.edgeNormalize()
		prober.meanShrink(shrink)
		prober=prober.clip(-pxs,-pxs,-pxs,fsize,fsize,fsize)

		ccf=targets.calcCCF(prober)
#		ccf.toCorner()
#		prober.writeImage("ccf.%d.mrc"%n)
		print ccf.Mean(),ccf.Max(),ccf.Sigma()
		n=n+1

		for i in range(0,fsize**3):
			if (ccf.valueAt(i,0,0)>ccf.Mean()+ccf.Sigma()*2.0) : loclist.append([-ccf.valueAt(i,0,0),i,d2r(aa[0]),d2r(aa[1]),d2r(phi)])

	loclist.sort()
	loclist=loclist[0:100]

for i in loclist:
	print i[0], coords(i[1],fsize)
