// focalpair.C 6/4/2001  Steve Ludtke
// Copyright 2001  Baylor College of Medicine

//N focalpair
//F Merges focal pairs into a single image with partial CTF correction
//T X
//1
//P <input 1>	First input (MRC only!)
//P <input 2>	Second input (MRC only!)
//P <output>	Combined output
//P [im1parm=<parm 1>]	Parameters of first image (from ctfit)
//P [im2parm=<parm 2>]	Parameters of second image
//P [im1=<ctfparm entry>]	Label from ctfparm.txt file
//P [im2=<ctfparm entry>]	Same for second image
//P sffile=<SF File>	Structure factor file (intensity vs s, 2 column)
//P [noampc]	Do not perform amplitude correction or Wiener filtration
//P [filt=<%>]	High-pass filter the output, 0-100 as a % of Fourier space
//U fpmerge jj1000.mrc jj1001.mrc jj1000m.mrc sffile=groel.xray
//D This program impliments focal pair merging for the purpose of combining the
//E low resolution information from a second exposure with the higher resolution
//E information from the first exposure. It performs partial CTF correction.
//E That is, it properly weights the two data sets during combination, and
//E performs phase flipping. However, it does not deconvolute the actual
//E CTF. Instead, it writes an output file containing the necessary information
//E for complete CTF correction at a later point.
//D The filt option (confusingly) specifies a filter radius as a percentatge.
//E A typical value for the filt option would be ~5, but this may vary for
//E different microscopes/specimens.

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include "XYData.h"

#define BOXAREA 1024
#define BOXPAD 128
#define BOXBIG (BOXAREA+2*BOXPAD)

float align(EMData *d1,EMData *d2);
void findAlign(EMData *d1, EMData *d2,float *rdxdy);

XYData SF;

main(int argc,char *argv[]) {
float parm1[11] = { 0,0,0,0,0,0,0,0,0,0,0 } ,parm2[11] = { 0,0,0,0,0,0,0,0,0,0,0 };
float *curve1,*curve2;
float filt=0;
int sx1,sy1,sx2,sy2,i;
int X,Y,noac=0,noali=0;
char stmp[30];
char *sffile;

appinit(argc,argv);
if (argc<4) appusage(argv[0]);

for (i=4; i<argc; i++) {	
	if (strncmp(argv[i],"sffile=",7)==0) sffile=argv[i]+7;
	else if (strncmp(argv[i],"filt=",5)==0) filt=atof(argv[i]+5);
	else if (strncmp(argv[i],"im1parm=",8)==0) sscanf(argv[i]+8,"%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f",parm1,parm1+1,parm1+2,parm1+3,parm1+4,parm1+5,parm1+6,parm1+7,parm1+8,parm1+9,parm1+10);
	else if (strncmp(argv[i],"im2parm=",8)==0) sscanf(argv[i]+8,"%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f",parm2,parm2+1,parm2+2,parm2+3,parm2+4,parm2+5,parm2+6,parm2+7,parm2+8,parm2+9,parm2+10);
	else if (strncmp(argv[i],"im1=",4)==0) {
		FILE *in = fopen("ctfparm.txt","r");
		char s1[256],s2[256],t[80];
		if (!in) error(ERR_EXIT,"Cannot open ctfparm.txt");
		s2[0]=0;
		while (fgets(s1,255,in)) {
			sscanf(s1,"%s",t);
			if (strcmp(t,argv[i]+4)==0) strcpy(s2,s1);
		}
		if (s2[0]==0) error(ERR_EXIT,"im1 label not found in ctfparm.txt");
		sscanf(s2,"%s %f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f",t,parm1,parm1+1,parm1+2,parm1+3,parm1+4,parm1+5,parm1+6,parm1+7,parm1+8,parm1+9,parm1+10);
		fclose(in);
	}
	else if (strncmp(argv[i],"im2=",4)==0) {
		FILE *in = fopen("ctfparm.txt","r");
		char s1[256],s2[256],t[80];
		if (!in) error(ERR_EXIT,"Cannot open ctfparm.txt");
		s2[0]=0;
		while (fgets(s1,255,in)) {
			sscanf(s1,"%s",t);
			if (strcmp(t,argv[i]+4)==0) strcpy(s2,s1);
		}
		if (s2[0]==0) error(ERR_EXIT,"im2 label not found in ctfparm.txt");
		sscanf(s2,"%s %f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f",t,parm2,parm2+1,parm2+2,parm2+3,parm2+4,parm2+5,parm2+6,parm2+7,parm2+8,parm2+9,parm2+10);
		fclose(in);
	}

	else if (strncmp(argv[i],"noampc",6)==0) noac=1;
	else if (strncmp(argv[i],"noali",5)==0) noali=1;
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);
LOG(Ref,argv[2],LOG_INFILE,NULL);

strcpy(stmp,"fpXXXXXX");
close(mkstemp(stmp));

char com[1024];
if (noali) sprintf(com,"cp %s %s",argv[2],stmp); 
else sprintf(com,"alignhuge %s %s %s",argv[2],argv[1],stmp);
puts(com);
run(com);


EMData *d1,*d2,*d3,*dflip1,*dflip2,*d1f,*d2f,output;
d1=new EMData;
d2=new EMData;
if (d1->readMRC(argv[1],1)) error(ERR_EXIT,"Error reading file1");
if (d2->readMRC(stmp,1)) error(ERR_EXIT,"Error reading file2");

if (parm1[0]==0) {
	char s[256];
	FILE *in=fopen("ctfparm.txt","r");
	while (fgets(s,255,in)) {
		if (strncmp(s,argv[1],strlen(argv[1]))==0) 
			sscanf(s+strlen(argv[1])," %f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f",parm1,parm1+1,parm1+2,parm1+3,parm1+4,parm1+5,parm1+6,parm1+7,parm1+8,parm1+9,parm1+10);
	}
	fclose(in);
}

if (parm2[0]==0) {
	char s[256];
	FILE *in=fopen("ctfparm.txt","r");
	while (fgets(s,255,in)) {
		if (strncmp(s,argv[2],strlen(argv[2]))==0) 
			sscanf(s+strlen(argv[2])," %f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f",parm2,parm2+1,parm2+2,parm2+3,parm2+4,parm2+5,parm2+6,parm2+7,parm2+8,parm2+9,parm2+10);
	}
	fclose(in);
}

// set up the structure factor data
if (SF.readFile(sffile)) { printf("Couldn't read structure factor file!\n"); exit(1); }
float *d=SF.getData();
for (i=1; i<SF.n()*2; i+=2) d[i]=(d[i]<=0)?-4.0:log10(d[i]);
SF.update();

sx1=d1->xSize();
sy1=d1->ySize();

sx2=d2->xSize();
sy2=d2->ySize();

printf("Image 1 is %d x %d\nImage 2 is %d x %d\n",sx1,sy1,sx2,sy2);

delete d1;
d1=new EMData;
d1->setSize(BOXBIG,BOXBIG);
d1->setCTF(parm1);
curve1=d1->ctfCurve(0);

// Set up the phase flipping map for image 1
dflip1=new EMData;
dflip1->setSize(BOXBIG+2,BOXBIG);
dflip1->setRI(1);
d=dflip1->getData();
int x,y;
i=0;
for (y=0; y<BOXBIG; y++) {
	for (x=0; x<BOXBIG/2+1; x++,i+=2) {
		int r=round((hypot(x,y-BOXBIG/2)*CTFOS));
		if (curve1[r]>=0) d[i]=1.0; else d[i]=-1.0;
		d[i+1]=d[i];
	}
}
dflip1->doneData();
dflip1->writeIMAGIC("flip.hed",0);

delete d2;
d2=new EMData;
d2->setSize(BOXBIG,BOXBIG);
d2->setCTF(parm2);
curve2=d2->ctfCurve(0);

// Set up the phase flipping map for image 2
dflip2=new EMData;
dflip2->setSize(BOXBIG+2,BOXBIG);
dflip2->setRI(1);
d=dflip2->getData();
i=0;
for (y=0; y<BOXBIG; y++) {
	for (x=0; x<BOXBIG/2+1; x++,i+=2) {
		int r=round((hypot(x,y-BOXBIG/2)*CTFOS));
		if (curve2[r]>=0) d[i]=1.0; else d[i]=-1.0;
		d[i+1]=d[i];
	}
}
dflip2->doneData();
dflip2->writeIMAGIC("flip.hed",1);

// Setup big output image
output.setSize(sx1,sy1);
output.zero();
printf("scale by %1.5g\n",parm1[2]/parm2[2]);

// MAIN LOOP
int cn;
int xstep,ystep;
xstep=floor((sx1-BOXPAD*2)/ceil((sx1-BOXPAD*2)/(float)BOXAREA))-1.0;
ystep=floor((sy1-BOXPAD*2)/ceil((sy1-BOXPAD*2)/(float)BOXAREA))-1.0;
printf("Step %d,%d\n",xstep,ystep);
int first=1;
for (Y=0; Y<sy1-BOXPAD-BOXAREA; Y+=ystep) {
	for (X=0; X<sx1-BOXPAD-BOXAREA; X+=xstep) {

printf("Clip at %d,%d\n",X,Y);

d1->readMRCArea(argv[1],X,Y,BOXBIG,BOXBIG);
d2->readMRCArea(stmp,X,Y,BOXBIG,BOXBIG);
d1->realFilter(60);
d2->realFilter(60);

// High pass filter
/*EMData *d1c=d1->copy(0,0);
EMData *d2c=d2->copy(0,0);
d1c->filter(-5.,BOXBIG/40.0,1);
d2c->filter(-5.,BOXBIG/40.0,1);
d1->subtract(d1c);
d2->subtract(d2c);
delete d1c;
delete d2c;*/
if (filt>0) {
	d1->filter(BOXBIG*filt/200.0,0,8);
	d2->filter(BOXBIG*filt/200.0,0,8);
}

//float mul=parm1[2]/parm2[2];
float mul=parm1[6]/parm2[6];
d2->multConst(mul);	// scale image 2 so image amplitude matches image 1
parm2[6]*=mul;
parm2[2]*=mul;
d1->setCTF(parm1);
d2->setCTF(parm2);
if (!noac) {
	d1->addConst(-d1->edgeMean());
	d2->addConst(-d2->edgeMean());
}
delete d3;

EMData *f;

// Fix the phases in image 1
f = d1->doFFT();
f->mult(dflip1);
d1->gimmeFFT();
delete d1;
d1=f->doIFT();
f->gimmeFFT();
delete f;

// Fix the phases in image 2
f = d2->doFFT();
f->mult(dflip2);
d2->gimmeFFT();
delete d2;
d2=f->doIFT();
f->gimmeFFT();
delete f;

List lst;
lst.addObject(d1);
lst.addObject(d2);

EMData tmp;
if (noac) {
	EMData ctfparm;
	tmp.makeAverageWT(&lst,&ctfparm,&SF);
//	tmp.makeAverageWT(&lst,NULL,NULL);
	if (first) {
		ctfparm.writeIMAGIC("ctfparm.hed",-1);
		cn=fileCount("ctfparm.hed")-1;
	}
}
else tmp.makeAverageCTFCW(&lst,&SF);
//tmp.filter(6,BOXAREA*4,1);

EMData *clip=tmp.clip(BOXPAD,BOXPAD,BOXAREA,BOXAREA);
//clip->addConst(-clip->edgeMean());
output.insertClip(clip,X+BOXPAD,Y+BOXPAD);
delete clip;
//delete d1;
//delete d2;
//delete d3;
	first=0;
	}
}

unlink(stmp);
char s[80];
if (noac) {
	sprintf(s,"!$ %d,%1.5g,ctfparm.hed",cn,parm1[10]);
	output.setCTFF(s);
}
output.writeMRC(argv[3]);
LOG(Ref,argv[3],LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);
}

