// make3drp.C 7/30/98 Steve Ludtke
// This program takes a set of projections (classes) with euler angles and constructs
// a 3d model in fourier space using a gaussian interpolation algorithm.
// This version also refines euler angles and is parallelized

// NOTE
// This version has been abandoned. Refinement in fourier space works poorly
// due to interpolation artifacts

#include <stdio.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include <pthread.h>
#include <math.h>

#define NITER 10

void *parallel(void *s);

List *data,*dataf,*dataf2;
int mode=5,it,im,goodbad=0,rsym=1,imc=0;
int *tstatus;
EMData *d2,*f,*out,*out2;
float prm,prs,prn,rec,thr;

main(int argc, char *argv[]) {
int mask=-1,pad=0,proc=1,th;
float sigm=3.0;
int i,j,n,ny,ny2,pu=0;
char *file="threed.mrc";
pthread_t *thread;
EMData *d;

appinit(argc,argv);

if (argc<2) { printf("make3drp\nWARNING: program no longer supported\nUsage: make3d <input file> [out=<outfile>] [proc=<# proc>]\
 [sym=<c4,icos,etc>] [pad=<padsize>] [mode=<1-6>] [mask=<rad>] [keep=<sigmamult>] [goodbad]\n"); exit(0); }

n=fileCount(argv[1]);
if (n<3) { printf("Input file too small ?!?!\n"); exit(1); }

for (i=2; i<argc; i++) {	
	if (strncmp(argv[i],"out=",4)==0) file=&argv[i][4];
	else if (strncmp(argv[i],"mode=",5)==0) mode=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"proc=",5)==0) proc=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"mask=",5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"keep=",5)==0) sigm=atof(&argv[i][5]);
	else if (strncmp(argv[i],"goodbad",7)==0) goodbad=1;
	else if (strncmp(argv[i],"sym=",4)==0) {
		if (argv[i][4]=='c') rsym=atoi(&argv[i][5]);
		else { printf("Unsupported symmetry\n"); exit(1); }
	}
	else if (strncmp(argv[i],"pad=",4)==0) pad=atoi(&argv[i][4]);
	else { printf("Unknown argument '%s'\n",argv[i]); exit(1); }
}

if (goodbad) {
	unlink("3drgood.hed");
	unlink("3drbad.hed");
	unlink("3drgood.img");
	unlink("3drbad.img");	
}

dataf=new List;
dataf2=new List;

// Read slices
printf("Read data\n");
data=readImages(argv[1],0,-1);
if (data->count()<3) {
	printf("Need at least 3 images!\n");
	exit(1);
}
d=(EMData *)data->objectAt(i);
ny=d->ySize();
if (pad) ny2=pad; else ny2=ny;

// prepare slices
printf("Transform slices\n");
for (i=0; i<data->count(); i++) {
	d=(EMData *)data->objectAt(i);
	if (d->NImg()<0) { delete d; continue; }
	if (d->NImg()==0) d->setNImg(1);
	d->normalize();
	d->applyMask(mask,0);

	d2=d->clip((ny-ny2)/2,(ny-ny2)/2,ny2,ny2);
	d2->toCorner();
	f=d2->doFFT();
	d2->gimmeFFT();
	dataf->addObject(f);
	f->setRAlign(d->Daz(),d->Dax(),d->Daz2());
	f->setNImg(d->NImg());

	d2->toCorner();
	d2->applyMask(0,20);		// not sure if this is good or bad
								// may improve phase residual calc.
	d2->toCorner();
	f=d2->doFFT();
	d2->gimmeFFT();
	dataf2->addObject(f);
	f->setRAlign(d->Daz(),d->Dax(),d->Daz2());
	f->setNImg(d->NImg());
	
	delete d2;
	delete d;	
}

thread=(pthread_t *)malloc(sizeof(pthread_t)*proc);
tstatus=(int *)malloc(sizeof(int)*proc);
for (j=0; j<proc; j++) {
	thread[j]=NULL;
	tstatus[j]=0;
}

out=new EMData;
out->setup4IS(ny2);
out2=NULL;

thr=PI/2.0;
for (it=0; it<NITER; it++) {
	
	imc=0;
	prm=prs=prn=rec=0;
	
// This is the parallel execution loop
	im=0;
	while (im<dataf->count()) {
		for (j=0; j<proc; j++) {
			if (tstatus[j]==0) {
//				printf("call %d %d\n",j,im);
				if (pthread_create(&thread[j],NULL,parallel,(void *)j)) {
					printf("Too many processes requested!\n");
					exit(1);
				}
				while (!tstatus[j]) usleep(50);
				im++;
//				printf("im++\n");
				break;
			}
		}

		usleep(100);

		// -1 means the thread is done
		for (j=0; j<proc; j++) {
			if (tstatus[j]==-1) {
				pthread_join(thread[j],NULL);
				tstatus[j]=0;
			}
		}
	}

	// now make sure all threads are joined
	for (j=0; j<proc; j++) {
		if (tstatus[j]!=0) {
			pthread_join(thread[j],NULL);
			tstatus[j]=0;
		}
	}
	
	if (prn) {
		prm/=prn;
		prs=sqrt(prs/prn-prm*prm);
		printf("%d. Phase residual Mean=%f   Sigma=%f   used %d images  %1.0f refined\n",it,prm*180.0/PI,prs*180.0/PI,imc,rec);
		thr=prm+prs*sigm;
	}

	if (out2) {
		out2->doneSlice();
		delete out2;
	}
	out2=out;
	out=new EMData;
	out->setup4IS(ny2);
}

out2->doneSlice();
out=out2->doIFT();
out->toCorner();

if (mode>=3&&mode<=5) out->applyMask(-1,10);
else if (mode==6) out->applyMask(-1,12);
else if (mode==2) out->applyMask(-1,11);
if (pad) {
	out=out->clip((ny2-ny)/2,(ny2-ny)/2,(ny2-ny)/2,ny,ny,ny);
}
out->applyMask(mask,0);
out->writeMRC(file);

}

void *parallel(void *s) {
int i=im,k,pn=(int)s;
float alt,az,phi,alt3,az3,phi3,rstep,pr,r,norm;
float prb,be[3];
EMData *d;

printf("thread %d %d\n",pn,i);
tstatus[pn]=1;
d=(EMData *)dataf->objectAt(i);
if (!d) { printf("Internal error\n"); exit(1); }
if (d->NImg()<=0) { tstatus[pn]=-1; return NULL; }

if (it<6) rstep=1.0*PI/180.0;
else if (it<8) { rstep=.5*PI/180.0; mode=5; }
else rstep=.25*PI/180.0;

//		if (it==9) mode=5;
		
if (it!=0) {
	d2=(EMData *)dataf2->objectAt(i);
	alt=d->Dax();
	az=d->Daz2();
	phi=d->Daz();
	prb=PI;
	for (alt3=alt-rstep*2.0; alt3<=alt+rstep*2.0; alt3+=rstep) {
		for (az3=az-rstep*2.0; az3<=az+rstep*2.0; az3+=rstep) {
			for (phi3=phi-rstep*2.0; phi3<=phi+rstep*2.0; phi3+=rstep) {
				r=out2->normSlice(d2,alt3,az3,phi3,&pr);
				if (pr<prb) { prb=pr; be[0]=alt3; be[1]=az3; be[2]=phi3; norm=r; }
			}
		}
	}
	d->setRAlign(be[2],be[0],be[1]);
	pr=prb;
	if (norm!=0) { d->multConst(1.0/norm); d2->multConst(1.0/norm); }
	prm+=pr;
	prs+=pr*pr;
	prn+=1.0;
	printf("%d/%d (%d)  %d\t%5.1f  %5.1f  %5.1f\t%f\t%f ",i,dataf->count(),pn,
		d2->NImg(),(d2->Dax()-alt)*180.0/PI,(d2->Daz2()-az)*180.0/PI,(d2->Daz()-phi)*180.0/PI,
		norm,pr*180.0/PI);
	if (d->Dax()!=alt || d->Daz2()!=az || d2->Daz()!=phi) rec++;
	if (pr>thr||norm<.01||norm>100.0) { d->setNImg(-1); printf(" X\n"); }
	else printf("\n");
}
else printf("%d/%d (%d) %d\t%5.1f  %5.1f  %5.1f\n",i,dataf->count(),pn,d->NImg(),
		d->Dax()*180.0/PI,d->Daz2()*180.0/PI,d->Daz()*180.0/PI); 
if (d->NImg()>0) {
	imc+=d->NImg();
	for (k=0; k<rsym; k++) {
		out->insertSlice(d,d->Dax(),d->Daz2()+k*2.0*PI/(float)rsym,			
			d->Daz(),mode,d->NImg());
	}
}
if (it==NITER-1 && goodbad) {
	if (d->NImg()>0) d->doIFT()->writeIMAGIC("3drgood.hed",-1);
	else d->doIFT()->writeIMAGIC("3drbad.hed",-1);
}

tstatus[pn]=-1;
return NULL;
}
