// proc2d.C 8/97  Steve Ludtke

//N proc2d
//F This program performs various processing operations on images and sets of images (WARNING: auto normalization removed)
//T LM
//1
//P <input file>	Source file
//P <output file>	Destination file, appends unless 'inplace' specified
//P [first=<n>]	Specify the first image in the input to process (0 - n-1)
//P [last=<n>]	Specify the last image in the input to process
//P [list=<list file>]	Works only on the image numbers contained in <list file>
//P [exclude=<list file>]	Excludes image numbers in <list file>
//P [inplace]	Output overwrites input, USE SAME FILENAME, DO NOT 'clip' images.
//P [edgenorm]	Output is always normalized, this will set 0 to the circular edge value rather than the overall mean
//P [norm[=<mean>,<sigma>]]	Normalize the result
//P [invert]	Inverts the density
//P [flip]	Vertical flip of image (inverts handedness)
//P [rot=<angle>]	Rotate by an angle (degrees)
//P [trans=<dx>,<dy>]	Translate
//P [center]	Center image using center of mass
//P [acfcenter]	Center image using CCF with 180 deg rotation
//P [scale=<sca>]	Scale by specified scaling factor. Clip must also be specified to change the dimensions of the output map
//P [clip=<newsize x&y>]	Define the output image size (always square)
//P [shrink=<n>]	Reduce an image size by an integral scaling factor, uses median filter. Clip is not required.
//P [meanshrink=<n>]	Same as shrink, but takes the average instead of the median. Clip is not required.
//P [apix=<A/pix>]	Specify the Angstr/pixel for S scaling
//P [lp=<filt r>]	Gaussian lowpass filter, r in angstroms if apix defined, pixels if undef
//P [hp=<filt r>]	Arctan highpass filter, r in angstroms if apix defined, pixels if undef
//P [rfilt=<type>[,<val1>[,<val2>[,<val3>]]]]	Apply one of a variety of real-space filters
//P [blfilt=<sigma1,sigma2,iter,localwidth>]	Apply bilateral filtering
//P [mask=<radius>]	Circular mask
//P [imask=<radius>]	Circular inner mask
//P [noisemask]	Fill the masked region with 1-sigma noise rather than zero
//P [mrc]	Output a set of MRC files rather than a single IMAGIC stack
//P [mrc8bit]	Output a set of 8-bit MRC files to conserve space
//P [pif]	Output to a PIF (Purdue Image Format) file
//P [spider]	Output a Spider stack file rather than an IMAGIC stack
//P [spider-single]	Output in individual Spider files rather than an IMAGIC file
//P [spiderswap]	Output a spider stack file in byte-swapped order (ie - PC vs SGI)
//P [spiderswap-single]	Output in individual Spider files in swapped order rather than an IMAGIC file
//P [pgm[=<low>,<hi>]]	Output a set of PGM images rather than an IMAGIC stack
//P [sharphp=<pixels>]	A highpass with a sharp cutoff (beware artifacts)
//P [norefs]	Skip any input images which are marked as references (usually used with classes.*)
//P [rotav]	Each image is rotationally averaged
//P	[average]	Averages all input images (without alignment) and writes a single (normalized) output image 
//P [split=n]	Splits the input file into a set of n output files
//P [ctfsplit]	Splits the input file into output files with the same CTF parameters
//P [interlv=<file 2 interleave>]	Specifies a 2nd input file. Output will be 2 files interleaved.
//P [sym=<Cn>]	Apply an n-fold rotational symmetry to each image
//P	[plt[=<plt file>]]	output the orientations in IMAGIC .plt file format	
//P [addnoise=<level>]	Add flatband gaussian noise with defined standard deviation
//P [randomize=<dr>,<da>,<flip>]	This will randomly rotate and translate each particle
//P [snrfilt=<SF file>]	Filter the images by the estimated SNR in each image.
//P [wiener=<SF file>]	Wiener filter the images using the estimated SNR with CTF amplitude correction.
//P [selfcl[=<steps>][,<mode>]]	Output file will be a 180x180 self-common lines map for each image.
//P	[radon]	Do Radon transform
//P [rfp]	Experimental
//P [mraprep]	Experimental
//P [phot]	Experimental
//P [rsub]	Experimental
//U proc2d 3244.img start.hed invert apix=5 lp=18
//U proc2d start.hed start.hed invert inplace
//D IMPORTANT NOTE: proc2d no longer automatically normalizes output images. Use the 'norm' or 'edgenorm' flag to re-enable this.
//D This program is used to perform a variety of generic 2D image processing tasks. Like all of the
//E EMAN programs, it will read any supported file format. By default it outputs to an Imagic file although
//E there are options to change this. Note that, by default, images are appended to the end of the output
//E file if it already exists. All images in one file MUST have the same size. ie - if you try to copy 50x50
//E images to the end of a file that already contains 100x100 images, an error will result. Note the 'first='
//E and 'last=' options allow a subset of the input file to be processed. The 'inplace' keyword may be used
//E to cause output images to overwrite the input images rather than being appended to the end of the file.
//D Most of the processing is explained in sufficient detail above, but
//E there are a few notes:
//E <UL>
//E <LI> The scale operation does NOT resize the image in pixels. Generally you want to specify both scale and clip.
//E <LI> If apix is not specified, all filters are specified in pixels in Fourier space. If apix
//E is specified, all filters are specified in Angstroms. ie - 'apix=4.5 lp=8' would filter
//E the image using a Gaussian in Fourier space with a 1/e width of 1/8 reciprocal angstroms.
//E <LI> The 'center' option generally doesn't produce very good results on noisy images. Use the 'cenalignint'
//E program instead.
//E <LI> You might notice that 'mask'ing an image often produces a mask that doesn't match the density just inside
//E the mask very well. The 'edgemean' value normalizes the images so the mean value matches the edge of the image
//E rather than the overall mean value of the image.
//E <LI> The self common-lines option takes two optional parameters. The first is the size of the self common-line image. For
//E example, 180 would produce 180x180 maps, meaning a 2 degree spacing. Also note that these maps have been undiagonalized,
//E meaning the normal diagonal line of 1's is along the x-axis. The mode determines how the common-lines are calculated:
//E 0 will do real-space calculations, 1 will do amplitude-weighted Fourier phase-error common-lines scaled and inverted  so higher is better, 2 will do amplitude-only Fourier common-lines
//E <li> The real-space filter option can perform a variety of filters:
//E <UL><li>0: x<val1 -> 0	(simple threshold) <li>1: x<val1 -> 0, then normalize
//E <li>2: x<val1 -> 0, x>val1 -> 1 <li>3: x>val1 -> x 0<x<val1 -> 0 x<0 -> val2, then normalize
//E <li>4: x -> fabs(x) <li>5: x towards zero by val1 set to zero if passes zero
//E <li>6: x!=0 ->1 <li>7: x<val1*maxval -> 0 <li>9: x -> (x+val1)*val2<li>10: x -> sqrt(x) <li>11: x -> exp(x/val1-val2) <li>12: x -> 2*(val1-x)
//E <li>15: x -> log(x) <li>16: x -> e^x<li>20: median filter, val1 is size<li>21: like 20, but takes std dev of box instead of median<li>22: like 20, but takes max value<li>50: Tries to remove linear gradients
//E <li>51: Vertical stripe filter to remove Zeiss artifacts<li>60:Fill zeroes at the edges with nearest non-zero value<li>61: Fill constant regions with zero<li>62: Eliminate beamstop in diffraction images (sigma,x center,y center) negative sigma will do a radial subtract as well
//E <li>102: Eliminate x,y pixels from the edge of the image<li>103: Map each value to the radius at that point (useful in proc3d as well)
//E </UL>

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include "XYData.h"

#define MAXMICROCTF	1000	// Max # different micrographs for ctfsplit

int main(int argc, char *argv[]) {
float lp=0,hp=0,hp2=0,ramp=0,anoise=0,scale=1.0,rot=0,dx=0,dy=0,apix=-1.0,amask=0;
int mask=0,noise=6,invert=0,center=0,acfcenter=0,rfp=0,split=1,norefs=0,phot=0,clip=-1,clipy=-1,mrc=0,edgenorm=0;
int i,j,k,l,n,n0=0,n1=-1,rotav=0,shrink=1,rsub=0,imask=-1,csym=0,inpl=0,spider=0,mraprep=0,norm=0,pgm=0;
int flip=0,rize=0,rizef=0,wiener=0,scl=0,sclmd=0,rfilt=-1,avgonly=0,ctfsplit=0,exclude=0;
float rizedx=0,rizeda=0,pgmlo=0,pgmhi=0,rft1=0,rft2=0,rft3=0,nmean=0,nsig=1.0;
char outfile[1024],*il=NULL;
float dfval[MAXMICROCTF],bval[MAXMICROCTF];
EMData *d=NULL,*e,*f,*average=NULL;
float *dat;
float sigma=1;
char *plt=NULL, spiderfile[256];
int  spidersingle=0, radon=0;
char *snrfilt=NULL,*listfile=NULL;
XYData SF;
int pif = 0;
 
int bliter=0, blwidth=0, xc=-1, yc=-1, zc=-1;
float blsigma1=0, blsigma2=0;

appinit(argc,argv);

if (argc<3) appusage(argv[0]);

for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"center",6)==0) center=1;
	else if (strncmp(argv[i],"acfcenter",9)==0) acfcenter=1;
	else if (strncmp(argv[i],"invert",6)==0) invert=1;
	else if (strncmp(argv[i],"phot",4)==0) phot=1;
	else if (strncmp(argv[i],"apix=",5)==0) apix=atof(&argv[i][5]);
	else if (strncmp(argv[i],"lp=",3)==0) lp=atof(argv[i]+3);
	else if (strncmp(argv[i],"hp=",3)==0) hp=atof(argv[i]+3);
	else if (strncmp(argv[i],"sharphp=",8)==0) hp2=atof(argv[i]+8);
	else if (strncmp(argv[i],"average",7)==0) avgonly=1;
	else if (strncmp(argv[i],"ramp=",5)==0) ramp=atof(argv[i]+5);
	else if (strncmp(argv[i],"rot=",4)==0) rot=atof(argv[i]+4);
	else if (strncmp(argv[i],"trans=",6)==0) sscanf(argv[i]+6,"%f,%f",&dx,&dy);
	else if (strncmp(argv[i],"mask=",5)==0) mask=(int)atof(&argv[i][5]);
	else if (strncmp(argv[i],"imask=",6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp(argv[i],"automask=",9)==0) amask=atof(&argv[i][9]);
	else if (strncmp(argv[i],"first=",6)==0) n0=atoi(argv[i]+6);
	else if (strncmp(argv[i],"list=",5)==0) listfile=argv[i]+5;
	else if (strncmp(argv[i],"exclude=",8)==0) { listfile=argv[i]+8; exclude=1; }
	else if (strncmp(argv[i],"scale=",6)==0) scale=atof(argv[i]+6);
	else if (strncmp(argv[i],"last=",5)==0) n1=atoi(argv[i]+5);
	else if (strncmp(argv[i],"rfp",3)==0) rfp=1;
	else if (strncmp(argv[i],"noisemask",9)==0) noise=1;
	else if (strncmp(argv[i],"mraprep",7)==0) mraprep=1;
	else if (strncmp(argv[i],"rfilt=",6)==0) {
		sscanf(argv[i]+6,"%d,%f,%f,%f",&rfilt,&rft1,&rft2,&rft3);
	}
	else if (strncmp(argv[i],"blfilt=",7)==0) {
		sscanf(argv[i]+7,"%f,%f,%d,%d",&blsigma1,&blsigma2,&bliter,&blwidth);
	}
	else if (strncmp(argv[i],"addnoise=",9)==0) anoise=atof(&argv[i][9]);
	else if (strncmp(argv[i],"split=",6)==0) split=atoi(argv[i]+6);
	else if (strncmp(argv[i],"ctfsplit",8)==0) ctfsplit=1;
	else if (strncmp(argv[i],"shrink=",7)==0) shrink=atoi(argv[i]+7);
	else if (strncmp(argv[i],"meanshrink=",11)==0) shrink=-atoi(argv[i]+11);
	else if (strncmp(argv[i],"norefs",6)==0) norefs=1;
	else if (strncmp(argv[i],"selfcl",6)==0) {
		if (argv[i][6]=='=') {
			sscanf(argv[i]+7,"%d,%d",&scl,&sclmd);
			scl/=2;
		}
		else scl=90;
	}
	else if (strncmp(argv[i],"clip=",5)==0) {
		sscanf(argv[i]+5,"%d,%d",&clip,&clipy);
		if (clipy<0) clipy=clip;
	}
	else if (strncmp(argv[i],"wiener=",7)==0) {
		wiener=1;
		snrfilt=argv[i]+7;
		if (SF.readFile(snrfilt)) error(ERR_EXIT,"Couldn't read structure factor file!");
	
		float *d0=SF.getData();
		for (j=1; j<SF.n()*2; j+=2) d0[j]=(d0[j]<=0)?-4.0:log10(d0[j]);
		SF.update();
	}
	else if (strncmp(argv[i],"snrfilt=",8)==0) {
		snrfilt=argv[i]+8;
		if (SF.readFile(snrfilt)) error(ERR_EXIT,"Couldn't read structure factor file!");
	
		float *d0=SF.getData();
		for (j=1; j<SF.n()*2; j+=2) d0[j]=(d0[j]<=0)?-4.0:log10(d0[j]);
		SF.update();
	}
	else if (strncmp(argv[i],"pgm",3)==0) {
		pgm=1;
		if (argv[i][3]=='=') sscanf(argv[i]+4,"%f,%f",&pgmlo,&pgmhi);
	}
	else if (strncmp(argv[i],"mrc8bit",7)==0) mrc=2;
	else if (strncmp(argv[i],"mrc",3)==0) mrc=1;
	else if (strncmp(argv[i],"pif",3)==0) pif=1;
	else if (strncmp(argv[i],"spider-single",13)==0) spidersingle=1;
	else if (strncmp(argv[i],"spiderswap-single",17)==0) spidersingle=2;
	else if (strncmp(argv[i],"spiderswap",10)==0) spider=2;
	else if (strncmp(argv[i],"spider",3)==0) spider=1;
	else if (strncmp(argv[i],"rotav",5)==0) rotav=1;
	else if (strncmp(argv[i],"inplace",7)==0) inpl=1;	
	else if (strncmp(argv[i],"rsub",4)==0) rsub=1;	
	else if (strncmp(argv[i],"flip",4)==0) flip=1;	
	else if (strncmp(argv[i],"interlv=",8)==0) il=argv[i]+8;
	else if (strncmp(argv[i],"edgenorm",8)==0) edgenorm=1;
	else if (strncmp(argv[i],"randomize=",10)==0) {
		rize=1;
		sscanf(&argv[i][10],"%f,%f,%d",&rizedx,&rizeda,&rizef);
		rizeda*=PI/180.0;
	}
	else if (strncmp(argv[i],"radon",5)==0) radon=1;
	else if (strncmp(argv[i],"plt=",4)==0) plt=&argv[i][4];
	else if (strncmp(argv[i],"norm",4)==0) {
		if (argv[i][4]=='=') {
			norm=2;
			sscanf(argv[i]+5,"%f,%f",&nmean,&nsig);
		}
		else norm=1;
	}
	else if (strncmp(argv[i],"sym=",4)==0) {
		if (tolower(argv[i][4])!='c') { printf("Only C symmetry currently supported!\n"); exit(1); }
		csym=atoi(&argv[i][5]);
	}
	else if (strncmp(argv[i],"usefs=",6)==0) ;
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

if(spidersingle && spider) { printf("Specify either \"spider\" or \"spider-single\", not both\n"); exit(1); }

if(spidersingle && mrc) { printf("Specify either \"mrc\" or \"spider-single\", not both\n"); exit(1); }

if ((spider?1:0) + (mrc?1:0) + pgm >1) { printf("Specify only 1 of spider, mrc or pgm\n"); exit(1); }


if ((mrc||pgm||spider||pif)&&avgonly) error(ERR_EXIT,"Average images can only be written to IMAGIC files."); 

if (norm && edgenorm) printf("Warning: norm and edgenorm specifcied\n");

if (clip>0 && !edgenorm) printf("Warning: suggest using edgenorm with clip\n");
if (split>1 && ctfsplit) error(ERR_EXIT,"Only one of split= and ctfsplit may be used.");

n=fileCount(argv[1]);
if (n<=n1 || n1<0) n1=n-1;

FILE* pltfp;
if(plt) pltfp=fopen(plt,"w");
char spiderformat[256];
sprintf(spiderformat,"%s%%0%dd.spi",argv[2],int(log10(i))+1);
		
if(sigma<=0)sigma=1;

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);

strcpy(outfile,argv[2]);

/*if (mraprep) {
	char ss[1024];
	for (i=0; i<10000; i++) {
		sprintf(ss,"%s%04d.lst",outfile,i);
		if (access(ss,F_OK)) break;
		unlink(ss);
//		sprintf(ss,"%s%04d.img",outfile,i);
//		unlink(ss);
	}
}*/

char *lf = NULL;
i=0;
if (listfile) {
	lf=(char *)calloc(n,1);
	if (exclude) memset(lf,1,n);
	FILE *in = fopen(listfile,"r");
	if (in) {
		char s[255];
		while (fgets(s,254,in)) {
			int v;
			if (s[0]=='#') continue;
			sscanf(s," %d",&v);
			if (v>=0 && v<n) {
				if (exclude) lf[v]=0;
				else lf[v]=1;
			}
			i++;
//printf("%d \n",v);
		}
		fclose(in);
	}
	else error(ERR_EXIT,"Couldn't read list file");
	printf("%d included\n",i);
}


d=new EMData;
EMData *ld = new EMData;
for (i=n0; i<=n1; i++) {
	if (lf && !lf[i]) continue;
	if (split>1) {
		strcpy(outfile,argv[2]);
		outfile[strlen(outfile)-4]=0;
		sprintf(outfile+strlen(outfile),".%02d.img",i%split);
	}
	if ((i+1)%10==0) { printf("\t%d complete\r",i+1); fflush(stdout); }
	if (ctfsplit) {
		delete ld;
		ld=d->copy(0,0);
	}
	d->readImage(argv[1],i);

	if(plt){	// out put the .plt file for use in IMAGIC 
		//fprintf(pltfp,"%f,%f,%f \n",d->getEuler()->alpha()*180/PI,d->getEuler()->beta()*180/PI, d->getEuler()->gamma()*180/PI);
		fprintf(pltfp,"%f,%f,%f \n",d->phi()*180/PI,d->alt()*180/PI, d->az()*180/PI);
		continue;
	}

	if (ctfsplit&&(i==n0 || d->CTFCmp(ld))) {
		for (j=1; j<ctfsplit; j++) if (dfval[j]==d->getCTF()[0] && bval[j]==d->getCTF()[1]) break;
		if (ctfsplit<=j) ctfsplit=j+1;
		dfval[j]=d->getCTF()[0];
		bval[j]=d->getCTF()[1];
		strcpy(outfile,argv[2]);
		outfile[strlen(outfile)-4]=0;
		sprintf(outfile+strlen(outfile),".%02d.img",j);
	}
	if (!finite(d->Sigma())) continue;
	if (d->Sigma()==0) printf("\nWarning! Sigma=0 for image %d\n",i);
	if (norefs && d->NImg()<0) continue;
	if (edgenorm) d->edgeNormalize(1);
	if (norm==1) d->normalize();
	else if (norm==2) { d->multConst(nsig/d->Sigma()); d->addConst(nmean-d->Mean()); }
	if (flip) d->vFlip();
	if (invert) d->invert();

// Phil's Butterworth mod
//	if (lp) d->filter(hp,lp==0?100000.0:lp,1);
//	if (hp) d->filter(hp,0,3);
	if (ramp) { d->filter(ramp,1.,6);}
	int y=d->ySize();


	if (snrfilt) {
		float *ctf;
		EMData *d2;
		if (wiener) ctf=d->ctfCurve(6,&SF);
		else ctf=d->ctfCurve(4,&SF);
		d->edgeNormalize();
		EMData *d3=d->clip(-d->xSize()/2,-d->ySize()/2,d->xSize()*2,d->ySize()*2);
		d2=d3->doFFT();
		d3->gimmeFFT();
		d2->applyRadFn(CTFOS*d->ySize()/2,0,2.0/CTFOS,ctf,0);
		delete d3;
		delete d;
		d3=d2->doIFT();
		d=d3->clip(d3->xSize()/4,d3->ySize()/4,d3->xSize()/2,d3->ySize()/2);
		d->setParent(NULL);
		delete d3;
		d2->gimmeFFT();
		delete d2;
		free(ctf);
	}

	float Xlp,Xhp,Xhp2;
	if (apix>0 && lp) Xlp=y*apix/lp; else Xlp=lp;
	if (apix>0 && hp) Xhp=y*apix/hp; else Xhp=hp;
	if (apix>0 && hp2) Xhp2=y*apix/hp2; else Xhp2=hp2;

	if (Xlp || Xhp) {  d->filter(Xhp==0?-10.0:Xhp,Xlp==0?100000.0:Xlp,1); }
	if (Xhp2) d->filter(Xhp2,100000.0,0);

	if (mask) d->applyMask(mask,noise);
	if (imask>0) d->applyMask(imask,5);
	
	if (amask) d->autoMask(amask,.1);
	
		// uses correlation with 180 deg rot for centering
	if (acfcenter) { d->transAlign(NULL,0,1,d->xSize()/4); d->rotateAndTranslate(); }

		// uses center of mass to center
	if (center) d->cmCenter(1); 
	if (phot) d->toCorner();
//	if (d->Sigma()==0 || !finite(d->Sigma())) continue;

	if (anoise) {
		dat=d->getData();
		for (j=0; j<d->xSize()*d->ySize(); j++) dat[j]+=grand(anoise,anoise/2);
		d->doneData();
	}

	if (rfp) { 
		e=d->makeRFP(); 
		e->writeIMAGIC("rfp.hed",-1); 
	}

	if (rot||dx||dy||rize) {
		Euler oldang=Euler(*d->getEuler());
		float dx0=d->Dx(),	dy0=d->Dy();
		d->setParent(NULL);
		d->setRAlign(rot*PI/180.0,0,0);
		d->setTAlign(dx,dy,0);
		if (rize) {
			if (rizeda>0) d->setRAlign(frand(-rizeda/2.0,rizeda/2.0));
			if (rizedx>0) d->setTAlign(grand(0,rizedx),grand(0,rizedx),0);
			if (rizef && random()%2) d->vFlip();
		}
		d->rotateAndTranslate();
		d->setRAlign(oldang.alt(),oldang.az(),oldang.phi()+rot*PI/180.0);
		d->setTAlign(dx0+dx,dy0+dy,0);
	}

	Euler ee = *d->getEuler();
	int ni = d->NImg();
	if (scale<1.0) { d->setRAlign(0,0,0); d->setTAlign(0,0,0); d->rotateAndTranslate(scale); }
	if (clip>0) {
		e=d->clip((d->xSize()-clip)/2,(d->ySize()-clipy)/2,clip,clipy);
		delete d;
		d=e;
	}
	if (scale>1.0) { d->setRAlign(0,0,0); d->setTAlign(0,0,0); d->rotateAndTranslate(scale); }
	d->setRAlign(&ee);
	d->setNImg(ni);

	if (shrink>1) d->medianShrink(shrink);
	else if (shrink<-1) d->meanShrink(-shrink);

	if (abs(shrink)>1 || scale!=1.0) {
		if (d->hasCTF()) {
			float *ctf=d->getCTF();
			ctf[10]*=fabs((float)shrink)/scale;
		}
	}

	if (rotav) {
		d->radialAverage();
	}


	if (csym>1) {
		f=d->copy();
		f->setParent(NULL);
		e=f->copy();
		for (j=1; j<csym; j++) {
			e->setRAlign(j*PI*2.0/(float)csym,0,0);
			e->rotateAndTranslate();
			d->add(e);
		}
		d->multConst(1.0/(float)csym);
		delete e;
		delete f;
	}

	if (rsub) {
		d->radialSubtract();
	}

	if (scl) {
		EMData *sc = new EMData;
		if (sclmd==0) sc->commonLinesR(d,d,scl,1);
		else {
//			EMData *e=d->clip(-d->xSize()/2,-d->ySize()/2,d->xSize()*2,d->ySize()*2);
			EMData *e=d->copy();
			e->toCorner();
			if (sclmd==1) {
				sc->commonLines(e,e,1,scl,1);
				sc->realFilter(9,-90.0,-1.0);
			}
			else if (sclmd==2) {
				sc->commonLines(e,e,2,scl,1);
			}
			else error(ERR_EXIT,"Invalid common-lines mode");
			delete e;
		}
		delete d;
		d=sc;
	}

	if(radon) {
		EMData *r=d->doRadon();
		delete d;
		d=r;
	}
		

	if (rfilt>=0) d->realFilter(rfilt,rft1,rft2,rft3);

	if (avgonly) {
		if (!average) average=d->copy(0,0);
		else average->add(d);
		continue;
	}

	if (bliter >0 && blwidth>0) {	// bilateral filtering
		d->BilateralFilter(blsigma1, blsigma2, bliter, blwidth);
	}

	if (mrc) {
		// single images go to outfile
		if (n1==0) {
			d->writeMRC(outfile,mrc==2?0:2);
		}
		// multi-images get numbered
		else {
			sprintf(outfile,"%03d.%s",i+100,argv[2]);
			d->writeMRC(outfile,mrc==2?0:2);
		}
	}
	else if (spidersingle) {
		sprintf(spiderfile,spiderformat,i);
		d->writeSingleSPIDER(spiderfile,spidersingle-1);
	}
	else if (pif) {
	        if (n1==0) {
		        d->writePIF(outfile);
		}
		else {
		        sprintf(outfile,"%03d.%s",i+100,argv[2]);
			d->writePIF(outfile);
		}
	}
	else if (pgm) {
		// single images go to outfile
		if (n1==0) {
			if (pgmlo>=pgmhi) d->writePGM(outfile,d->Min(),d->Max());
			else d->writePGM(outfile,pgmlo,pgmhi);
		}
		// multi-images get numbered
		else {
			sprintf(outfile,"%03d.%s",i+100,argv[2]);
			if (pgmlo>=pgmhi) d->writePGM(outfile,d->Min(),d->Max());
			else d->writePGM(outfile,pgmlo,pgmhi);
		}
	}
	else if (spider) {
		if (spider==1) d->writeSPIDER(outfile,-1);
		else d->writeSPIDER(outfile,-1,1);	// write byte-swapped
	}
	else {
		if (inpl) d->writeIMAGIC(outfile,i);
		else if (mraprep) {
			char nf[1024];
			sprintf(nf,"%s%04d.lst",outfile,i);
			d->writeLST(nf,0);
		}
		else d->writeIMAGIC(outfile,-1);
	}

	// interleaved images not processed
	if (il) {
		d->readImage(il,i);
		d->writeIMAGIC(outfile,-1);
	}

//	if (rfp) delete e;	// auotmatically freed by emdata
}

if (average) {
	average->setNImg(n1-n0+1);
	average->normalize();
	average->writeIMAGIC(outfile,-1);
}

LOG(Ref,argv[2],LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);
printf("%d images\n",fileCount(argv[2]));
}
