// speedtest.C  8/99  Steve Ludtke

//N speedtest
//F This program runs a set of speed tests on the current machine.
//T LM
//U speedtest
//D This program runs a set of speed tests in the current computer. It should
//E have at least 1 unloaded processor, and minimal i/o occuring when this
//E is run. It will give a single value which should be generally proportional
//E to how fast refine will run on a given computer. It is a 'real world' test,
//E in that it simulates a variety of actual operations performed by a 
//E refinement procedure. Don't compare values given by speed test in
//E different versions of EMAN, since the underlying routines may be
//E improved with time.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include <time.h>

#define NTT 200
#define CPS	(CLOCKS_PER_SEC)
#define SIZE	100		// 100 is correct for calibrated comparison

main(int argc, char *argv[]) {
int i,j,k;
clock_t t1,t2;
EMData *data[NTT],*tmp;
float *d,ti;
int slow=0;

if (argc>1 && strncmp(argv[1],"slowest",7)==0) slow=2;
else if (argc>1 && strncmp(argv[1],"slow",4)==0) slow=1;

printf("This could take a few minutes. Please be patient.\n");

// initialize 10 megs of data
printf("Initializing\n");
for (i=0; i<NTT; i++) {
	data[i]=new EMData;
	data[i]->setSize(SIZE,SIZE,1);
	d=data[i]->getData();
	
	for (j=0; j<SIZE*SIZE; j++) d[j]=grand(0,1.0);
	
	data[i]->doneData();
}

t1=clock();
for (i=0; i<3; i++) {
	for (j=5; j<(slow==2?NTT/10:NTT); j++) {
		if (slow==2) tmp=data[i]->RTFAlignSlowest(data[j],NULL,SIZE/8); 
		else if (slow==1) tmp=data[i]->RTFAlignSlow(data[j],NULL,SIZE/8); 
		else tmp=data[i]->RTFAlign(data[j]);
		delete tmp;
		putchar('.'); fflush(stdout);
	}
putchar('\n');
}
t2=clock();

ti=(t2-t1)/(float)CPS;
if (slow==2) ti*=10.0;
printf("\nFor comparison, one 200Mhz R10k processor on an SGI Onyx2 has a sf of 190\n");
printf("A  250Mhz R10k Octane has a SF of 260\n");
printf("A  PIII 800 has a sf of 260\n");
printf("A Xeon 2600mhz on a 4-way shared memory machine with all processors active has a sf of 380\n");
printf("A Xeon 2600mhz on a 4-way shared memory machine with 2 processors active has a sf of 400\n");
printf("A SGI Fuel, R14000 @ 500 Mhz has a SF of 500\n");
printf("An Athlon XP1900+ (1600mhz) with DDR266 SDRAM has a sf of 600\n");
printf("A Xeon 2600mhz (Redhat 7.3, gcc 2.96) has a sf of 610\n");
printf("An Athlon XP1700+ (1466mhz) with gcc3.2 and FFTWGEL has a sf of 730\n");
printf("\nYour machines speed factor = %1.1f\n",10000.0/ti);

}
