// xtalcen.C 4/2002		Steve Ludtke

//N xtalcen
//F This program will take a diffraction pattern, locate the center, remove the beamstop and do a background subtraction
//T LM
//1
//P <input file>
//P [out=<output name>]	Optional output image
//P [center=<x>,<y>]	No automatic centering
//P [filtrad=<rad>]	Radius of background elimination filter for centering, should be larger than peak size, default 10
//P [bgsub]	Do a background subtraction
//P [zerothr=<sigma mult>]	Threshold for zeroed data as a sigma multiplier, default 1.5
//U xtalcen edp1.mrc out=edp1a.mrc bgsub
//D	This program will find the center of a diffraction mode diffraction pattern, mask out the beamstop
//D and do an optional background subtraction.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
EMData d1,*d2,*d3;
int i,cenx=-1,ceny=-1,filtrad=10,bgsub=0;
float zerothr=1.5;
char *out=NULL;

appinit(argc,argv);

if (argc<2) appusage(argv[0]);

for (i=2; i<argc; i++) {	
	if (strncmp(argv[i],"center=",7)==0) sscanf(argv[i]+7,"%d,%d",&cenx,&ceny);
	else if (strncmp(argv[i],"filtrad=",8)==0) filtrad=atoi(argv[i]+8);
	else if (strncmp(argv[i],"out=",4)==0) out=argv[i]+4;
	else if (strncmp(argv[i],"bgsub",8)==0) bgsub=1;
	else if (strncmp(argv[i],"zerothr=",8)==0) zerothr=atof(argv[i]+8);
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

d1.readImage(argv[1],0);
int xs=d1.xSize();
int ys=d1.ySize();

if (cenx<0||ceny<0) {
	d2=d1.clip(xs/2-xs/8,ys/2-ys/8,xs/4,ys/4);
	d3=d2->copy(0,0);

	d3->filter(-10,xs/(2*filtrad),1);
	d2->subtract(d3);

	d2->writeMRC("cenref.mrc");
	d3=d2->calcCCF(NULL);
	d3->toCorner();
	d3->writeMRC("cenccf.mrc");
	
	cenx=((d3->MaxLoc()%(xs/4))-xs/8)/2+xs/2;
	ceny=((d3->MaxLoc()/(xs/4))-ys/8)/2+ys/2;

	delete d2;
	delete d3;
}
printf("Center: %d,%d\n",cenx,ceny);

if (!out) exit(0);

if (bgsub) d1.realFilter(62,-zerothr,(float)cenx,(float)ceny);
else d1.realFilter(62,zerothr,(float)cenx,(float)ceny);

d1.setTAlign(cenx,ceny);
d1.writeMRC(out);
printf("Write %s\n",out);
}
